@@ -2,5 +2,5 @@
 
 # Calls pyspread from top level folder of extracted tarball
 
-export PYTHONPATH=./pyspread
+export PYTHONPATH=$PYTHONPATH:./pyspread
 ./pyspread/pyspread $@
