@@ -36,11 +36,10 @@ Provides:
   4. TableTabActionsMixin: Mixin for TableActions
   5. TableActions: Actions which affect table
   6. MacroActions: Actions on macros
-  7. UnRedoActions: Actions on the undo redo system
-  8. GridActions: Actions on the grid as a whole
-  9. SelectionActions: Actions on the grid selection
-  10. FindActions: Actions for finding and replacing
-  11. AllGridActions: All grid actions as a bundle
+  7. GridActions: Actions on the grid as a whole
+  8. SelectionActions: Actions on the grid selection
+  9. FindActions: Actions for finding and replacing
+  10. AllGridActions: All grid actions as a bundle
 
 
 """
@@ -306,7 +305,6 @@ class FileActions(Actions):
         self.code_array.macros = ""
 
         # Clear caches
-        self.code_array.unredo.reset()
         self.code_array.result_cache.clear()
 
         # Clear globals
@@ -368,8 +366,10 @@ class FileActions(Actions):
                 # Make loading safe
                 self.approve(filepath)
 
-                # Disable undo
-                self.grid.code_array.unredo.active = True
+                if xlrd is None:
+                    interface_errors = (ValueError, )
+                else:
+                    interface_errors = (ValueError, xlrd.biffh.XLRDError)
 
                 try:
                     wx.BeginBusyCursor()
@@ -377,8 +377,10 @@ class FileActions(Actions):
                     self.clear()
                     interface = Interface(self.grid.code_array, infile)
                     interface.to_code_array()
+                    self.grid.main_window.macro_panel.codetext_ctrl.SetText(
+                        self.grid.code_array.macros)
 
-                except (ValueError, xlrd.biffh.XLRDError), err:
+                except interface_errors, err:
                     post_command_event(self.main_window, self.StatusBarMsg,
                                        text=str(err))
 
@@ -392,9 +394,6 @@ class FileActions(Actions):
                 # Execute macros
                 self.main_window.actions.execute_macros()
 
-                # Enable undo again
-                self.grid.code_array.unredo.active = False
-
                 self.grid.GetTable().ResetView()
                 self.grid.ForceRefresh()
 
@@ -473,8 +472,7 @@ class FileActions(Actions):
 
         # Mark content as unchanged
         try:
-            post_command_event(self.main_window, self.ContentChangedMsg,
-                               changed=False)
+            post_command_event(self.main_window, self.ContentChangedMsg)
         except TypeError:
             # The main window does not exist any more
             pass
@@ -596,7 +594,13 @@ class FileActions(Actions):
                 pass
 
         else:
-            self.sign_file(filepath)
+            try:
+                self.sign_file(filepath)
+
+            except ValueError, err:
+                msg = "Signing file failed. " + unicode(err)
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=msg)
 
     def save(self, event):
         """Saves a file that is specified in event.attr
@@ -664,8 +668,7 @@ class TableRowActionsMixin(Actions):
         """Sets row height and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -680,8 +683,7 @@ class TableRowActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -691,8 +693,7 @@ class TableRowActionsMixin(Actions):
         """Deletes no_rows rows and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -711,8 +712,7 @@ class TableColumnActionsMixin(Actions):
         """Sets column width and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -727,8 +727,7 @@ class TableColumnActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -738,8 +737,7 @@ class TableColumnActionsMixin(Actions):
         """Deletes no_cols column and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         tab = self.grid.current_table
 
@@ -762,8 +760,7 @@ class TableTabActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         self.code_array.insert(tab, no_tabs, axis=2)
 
@@ -775,8 +772,7 @@ class TableTabActionsMixin(Actions):
         """Deletes no_tabs tabs and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         try:
             self.code_array.delete(tab, no_tabs, axis=2)
@@ -929,11 +925,7 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
                     key = target_row, target_col, tl_tab
 
                     try:
-                        # Set cell but do not mark unredo
-                        # before pasting is finished
-
-                        self.grid.code_array.__setitem__(key, cell_data,
-                                                         mark_unredo=False)
+                        CellActions.set_code(self, key, cell_data)
                         no_pasted_cells += 1
                     except KeyError:
                         pass
@@ -944,10 +936,6 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         else:
             self._show_final_paste_message(tl_key, no_pasted_cells)
 
-        if no_pasted_cells:
-            # If cells have been pasted mark unredo operation
-            self.grid.code_array.unredo.mark()
-
         self.pasting = False
 
     def selection_paste_data_gen(self, selection, data, freq=None):
@@ -1006,8 +994,7 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         selection = self.get_selection()
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         if selection:
             # There is a selection.  Paste into it
@@ -1020,16 +1007,17 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         """Grid shape change event handler, marks content as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
-        self.grid.code_array.shape = shape
+        self.code_array.shape = shape
 
         # Update TableChoiceIntCtrl
         post_command_event(self.main_window, self.ResizeGridMsg, shape=shape)
 
+        # Change grid table dimensions
+        self.grid.GetTable().ResetView()
+
         # Clear caches
-        self.code_array.unredo.reset()
         self.code_array.result_cache.clear()
 
     def replace_cells(self, key, sorted_row_idxs):
@@ -1052,13 +1040,10 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
                     del_keys.append((__row, __col, __tab))
 
         for key in del_keys:
-            self.grid.code_array.pop(key, mark_unredo=False)
+            self.grid.code_array.pop(key)
 
         for key in new_keys:
-            self.grid.code_array.__setitem__(key, new_keys[key],
-                                             mark_unredo=False)
-
-        self.grid.code_array.unredo.mark()
+            CellActions.set_code(self, key, new_keys[key])
 
     def sort_ascending(self, key):
         """Sorts selection (or grid if none) corresponding to column of key"""
@@ -1095,28 +1080,6 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         self.grid.ForceRefresh()
 
 
-class UnRedoActions(Actions):
-    """Undo and redo operations"""
-
-    def undo(self):
-        """Calls undo in model.code_array.unredo, marks content as changed"""
-
-        # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
-
-        self.grid.code_array.unredo.undo()
-
-    def redo(self):
-        """Calls redo in model.code_array.unredo, marks content as changed"""
-
-        # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
-
-        self.grid.code_array.unredo.redo()
-
-
 class GridActions(Actions):
     """Grid level grid actions"""
 
@@ -1157,8 +1120,10 @@ class GridActions(Actions):
         for row, tab in self.code_array.row_heights:
             if tab == self.grid.current_table and \
                row < self.grid.code_array.shape[0]:
-                zoomed_row_size = \
-                    self.code_array.row_heights[(row, tab)] * zoom
+                base_row_width = self.code_array.row_heights[(row, tab)]
+                if base_row_width is None:
+                    base_row_width = self.grid.GetDefaultRowSize()
+                zoomed_row_size = base_row_width * zoom
                 self.grid.SetRowSize(row, zoomed_row_size)
 
     def _zoom_cols(self, zoom):
@@ -1171,7 +1136,10 @@ class GridActions(Actions):
         for col, tab in self.code_array.col_widths:
             if tab == self.grid.current_table and \
                col < self.grid.code_array.shape[1]:
-                zoomed_col_size = self.code_array.col_widths[(col, tab)] * zoom
+                base_col_width = self.code_array.col_widths[(col, tab)]
+                if base_col_width is None:
+                    base_col_width = self.grid.GetDefaultColSize()
+                zoomed_col_size = base_col_width * zoom
                 self.grid.SetColSize(col, zoomed_col_size)
 
     def _zoom_labels(self, zoom):
@@ -1490,7 +1458,8 @@ class GridActions(Actions):
             self.grid._last_selected_cell = row, col, self.grid.current_table
 
         if not (row is None and col is None):
-            self.grid.MakeCellVisible(row, col)
+            if not self.grid.IsVisible(row, col, wholeCellVisible=True):
+                self.grid.MakeCellVisible(row, col)
             self.grid.SetGridCursor(row, col)
 
     cursor = property(get_cursor, set_cursor)
@@ -1553,7 +1522,7 @@ class SelectionActions(Actions):
                 for col in xrange(col_slc.start, col_slc.stop, col_slc.step):
                     self.select_cell(row, col, add_to_selected=True)
 
-    def delete_selection(self, selection=None, mark_unredo=True):
+    def delete_selection(self, selection=None):
         """Deletes selection, marks content as changed
 
         If selection is None then the current grid selection is used.
@@ -1563,14 +1532,11 @@ class SelectionActions(Actions):
 
         selection: Selection, defaults to None
         \tSelection that shall be deleted
-        mark_unredo: Boolean, defaults to True
-        \tIf True deletion is marked as separate undo step
 
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         if selection is None:
             selection = self.get_selection()
@@ -1579,11 +1545,7 @@ class SelectionActions(Actions):
 
         for row, col, tab in self.grid.code_array.dict_grid.keys():
             if tab == current_table and (row, col) in selection:
-                self.grid.actions.delete_cell((row, col, tab),
-                                              mark_unredo=False)
-
-        if mark_unredo:
-            self.grid.code_array.unredo.mark()
+                self.grid.actions.delete_cell((row, col, tab))
 
         self.grid.code_array.result_cache.clear()
 
@@ -1613,10 +1575,7 @@ class SelectionActions(Actions):
         current_table = self.grid.current_table
         for row, col, tab in self.grid.code_array.dict_grid.keys():
             if tab == current_table and (row, col) in selection:
-                self.grid.actions.quote_code((row, col, tab),
-                                             mark_unredo=False)
-
-        self.grid.code_array.unredo.mark()
+                self.grid.actions.quote_code((row, col, tab))
 
         self.grid.code_array.result_cache.clear()
 
@@ -1647,12 +1606,6 @@ class SelectionActions(Actions):
         post_command_event(self.main_window, self.StatusBarMsg,
                            text=statustext)
 
-    def _shifted_merge_area(self, merge_area, rows, cols):
-        """Shifts merge area by rows and cols"""
-
-        top, left, bottom, right = merge_area
-        return top + rows, left + cols, bottom + rows, right + cols
-
     def copy_format(self):
         """Copies the format of the selected cells to the Clipboard
 
@@ -1679,13 +1632,12 @@ class SelectionActions(Actions):
         for __selection, table, attrs in cell_attributes:
             if tab == table:
                 new_selection = selection & __selection
-                new_shifted_selection = new_selection.shifted(-top, -left)
-                if "merge_area" in attrs and attrs["merge_area"]:
-                    attrs["merge_area"] = \
-                        self._shifted_merge_area(attrs["merge_area"],
-                                                 -top, -left)
-                new_cell_attributes.append(
-                    (new_shifted_selection.parameters, table, attrs))
+                if new_selection:
+                    new_shifted_selection = new_selection.shifted(-top, -left)
+                    if "merge_area" not in attrs:
+                        selection_params = new_shifted_selection.parameters
+                        cellattribute = selection_params, table, attrs
+                        new_cell_attributes.append(cellattribute)
 
         # Rows
 
@@ -1743,16 +1695,14 @@ class SelectionActions(Actions):
 
         # Cell attributes
 
-        for ca_ele in ca:
-            base_selection = Selection(*ca_ele[0])
+        for selection_params, tab, attrs in ca:
+            base_selection = Selection(*selection_params)
             shifted_selection = base_selection.shifted(row, col)
-            attrs = ca_ele[2]
-            if "merge_area" in attrs and attrs["merge_area"]:
-                attrs["merge_area"] = \
-                    self._shifted_merge_area(attrs["merge_area"], row, col)
-
-            new_cell_attribute = shifted_selection, tab, attrs
-            cell_attributes.undoable_append(new_cell_attribute)
+            if "merge_area" not in attrs:
+                # Do not paste merge areas because this may have
+                # inintended consequences for existing merge areas
+                new_cell_attribute = shifted_selection, tab, attrs
+                cell_attributes.append(new_cell_attribute)
 
         # Row heights
         row_heights = self.grid.code_array.row_heights
@@ -1853,8 +1803,7 @@ class FindActions(Actions):
 
         """
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         for findpos in findpositions:
             old_code = self.grid.code_array(findpos)
@@ -1888,8 +1837,7 @@ class FindActions(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         old_code = self.grid.code_array(findpos)
         new_code = old_code.replace(find_string, replace_string)
@@ -1903,15 +1851,15 @@ class FindActions(Actions):
         post_command_event(self.main_window, self.StatusBarMsg,
                            text=statustext)
 
+        self.grid.ForceRefresh()
 
-class AllGridActions(FileActions, TableActions, UnRedoActions,
+class AllGridActions(FileActions, TableActions,
                      GridActions, SelectionActions, FindActions, CellActions):
     """All grid actions as a bundle"""
 
     def __init__(self, grid):
         FileActions.__init__(self, grid)
         TableActions.__init__(self, grid)
-        UnRedoActions.__init__(self, grid)
         GridActions.__init__(self, grid)
         SelectionActions.__init__(self, grid)
         FindActions.__init__(self, grid)
