@@ -128,8 +128,7 @@ class ExchangeActions(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         if filterindex == 0:
             # CSV import option choice
@@ -673,6 +672,10 @@ class ClipboardActions(Actions):
 
         parameters = interfaces.get_pasteas_parameters_from_user(obj)
 
+        if parameters is None:
+            # Dialog aborted
+            return
+
         paste_data = self._get_pasteas_data(parameters["dim"], obj)
 
         if parameters["transpose"]:
@@ -693,8 +696,7 @@ class MacroActions(Actions):
         """Executes macros and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, self.ContentChangedMsg,
-                           changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg)
 
         (result, err) = self.grid.code_array.execute_macros()
 
@@ -722,12 +724,13 @@ class MacroActions(Actions):
                 post_command_event(self.main_window, self.SafeModeEntryMsg)
 
                 # Mark content as changed
-                post_command_event(self.main_window, self.ContentChangedMsg,
-                                   changed=True)
+                post_command_event(self.main_window, self.ContentChangedMsg)
 
                 macrocode = macro_infile.read()
 
                 self.grid.code_array.macros += "\n" + macrocode.strip("\n")
+                self.grid.main_window.macro_panel.codetext_ctrl.SetText(
+                    self.grid.code_array.macros)
 
         except IOError:
             msg = _("Error opening file {filepath}.").format(filepath=filepath)
@@ -741,8 +744,8 @@ class MacroActions(Actions):
 
         # Mark content as changed
         try:
-            post_command_event(self.main_window, self.ContentChangedMsg,
-                               changed=True)
+            post_command_event(self.main_window, self.ContentChangedMsg)
+
         except TypeError:
             # The main window does not exist any more
             pass
