@@ -50,8 +50,9 @@ sys.path.insert(0, TESTPATH + (os.sep + os.pardir) * 2)
 from src.gui._main_window import MainWindow
 from src.lib.selection import Selection
 
-from src.lib.testlib import params, pytest_generate_tests, unredo_test
+from src.lib.testlib import params, pytest_generate_tests, undo_test
 from src.lib.testlib import basic_setup_test, restore_basic_grid
+from src.lib.undo import stack as undo_stack
 
 from src.gui._events import *
 
@@ -157,18 +158,18 @@ class TestFileActions(object):
         # Test if safe_mode is correctly set for invalid sig
         self.grid.actions.approve(self.filename_invalid_sig)
 
-        assert self.grid.GetTable().data_array.safe_mode
+        assert self.grid.code_array.safe_mode
 
         # Test if safe_mode is correctly set for valid sig
 
         self.grid.actions.approve(self.filename_valid_sig)
 
-        assert not self.grid.GetTable().data_array.safe_mode
+        assert not self.grid.code_array.safe_mode
 
         # Test if safe_mode is correctly set for missing sig
         self.grid.actions.approve(self.filename_no_sig)
 
-        assert self.grid.GetTable().data_array.safe_mode
+        assert self.grid.code_array.safe_mode
 
         # Test if safe_mode is correctly set for io-error sig
 
@@ -177,7 +178,7 @@ class TestFileActions(object):
 
         self.grid.actions.approve(self.filename_not_permitted)
 
-        assert self.grid.GetTable().data_array.safe_mode
+        assert self.grid.code_array.safe_mode
 
         os.chmod(self.filename_not_permitted, 0644)
         os.chmod(self.filename_not_permitted + ".sig", 0644)
@@ -239,12 +240,6 @@ class TestFileActions(object):
         col_widths = self.grid.code_array.col_widths
         assert len(col_widths) == 0
 
-        # Undo and redo
-        undolist = self.grid.code_array.unredo.undolist
-        redolist = self.grid.code_array.unredo.redolist
-        assert undolist == []
-        assert redolist == []
-
         # Caches
 
         # Clear self.grid again because lookup is added in resultcache
@@ -277,37 +272,37 @@ class TestFileActions(object):
         event.attr["filepath"] = self.filename_empty
         assert not self.grid.actions.open(event)
 
-        assert self.grid.GetTable().data_array.safe_mode  # sig is also empty
+        assert self.grid.code_array.safe_mode  # sig is also empty
 
         if gnupg is not None:
             # Test invalid sig files
             event.attr["filepath"] = self.filename_invalid_sig
             self.grid.actions.open(event)
 
-            assert self.grid.GetTable().data_array.safe_mode
+            assert self.grid.code_array.safe_mode
 
             # Test file with sig
             event.attr["filepath"] = self.filename_valid_sig
             self.grid.actions.open(event)
 
-            assert not self.grid.GetTable().data_array.safe_mode
+            assert not self.grid.code_array.safe_mode
 
             # Test file without sig
             event.attr["filepath"] = self.filename_no_sig
             self.grid.actions.open(event)
 
-            assert self.grid.GetTable().data_array.safe_mode
+            assert self.grid.code_array.safe_mode
 
             # Test self.grid size for valid file
             event.attr["filepath"] = self.filename_gridsize
             self.grid.actions.open(event)
 
-            new_shape = self.grid.GetTable().data_array.shape
+            new_shape = self.grid.code_array.shape
             assert new_shape == (1000, 100, 10)
 
             # Test self.grid content for valid file
             assert not self.grid.code_array.safe_mode
-            assert self.grid.GetTable().data_array[0, 0, 0] == "test4"
+            assert self.grid.code_array((0, 0, 0)) == '"test4"'
 
     def test_save(self):
         """Tests save functionality"""
@@ -399,7 +394,7 @@ class TestTableRowActionsMixins(object):
         assert row_heights[row, tab] == height
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_insert_rows = [
         {'row': -1, 'no_rows': 0, 'test_key': (0, 0, 0), 'test_val': "'Test'"},
@@ -420,7 +415,7 @@ class TestTableRowActionsMixins(object):
                          test_val, row, no_rows=no_rows)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_delete_rows = [
         {'row': 0, 'no_rows': 0, 'test_key': (0, 0, 0), 'test_val': "'Test'"},
@@ -442,7 +437,7 @@ class TestTableRowActionsMixins(object):
                          test_val, row, no_rows=no_rows)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
 
 class TestTableColumnActionsMixin(object):
@@ -468,7 +463,7 @@ class TestTableColumnActionsMixin(object):
         assert col_widths[col, tab] == width
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_set_col_width_selection = [
         {'cursorCol': 1, 'tab': 0, 'width': 7,
@@ -571,7 +566,7 @@ class TestTableColumnActionsMixin(object):
                          test_val, col, no_cols=no_cols)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_delete_cols = [
         {'col': -1, 'no_cols': 0, 'test_key': (0, 0, 0), 'test_val': "'Test'"},
@@ -593,7 +588,7 @@ class TestTableColumnActionsMixin(object):
                          test_val, col, no_cols=no_cols)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
 
 class TestTableTabActionsMixin(object):
@@ -621,7 +616,7 @@ class TestTableTabActionsMixin(object):
                          test_val, tab, no_tabs=no_tabs)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_delete_tabs = [
         {'tab': 0, 'no_tabs': 0, 'test_key': (0, 0, 0), 'test_val': "'Test'"},
@@ -640,7 +635,7 @@ class TestTableTabActionsMixin(object):
                          test_val, tab, no_tabs=no_tabs)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
 
 class TestTableActions(object):
@@ -694,7 +689,7 @@ class TestTableActions(object):
                          test_val, tl_cell, data)
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_change_grid_shape = [
         {'shape': (1, 1, 1)},
@@ -719,7 +714,7 @@ class TestTableActions(object):
         assert self.grid.code_array(br_key) is None
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_replace_cells = [
         {'key': (0, 0, 0), 'sorted_row_idxs': [1, 0, 2, 3, 4, 5, 6, 7, 8, 9],
@@ -764,7 +759,7 @@ class TestTableActions(object):
         assert self.grid.code_array(res_key) == res
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     param_sort_ascending = [
         {'key': (0, 0, 0), 'selection': Selection([], [], [], [], []),
@@ -808,7 +803,7 @@ class TestTableActions(object):
             assert self.grid.code_array(res_key) == res
 
             # Test equality of code_array after undo and subsequent redo
-            unredo_test(self.grid)
+            undo_test(self.grid)
 
         except TypeError:
             assert res == 'fail'
@@ -855,41 +850,12 @@ class TestTableActions(object):
             assert self.grid.code_array(res_key) == res
 
             # Test equality of code_array after undo and subsequent redo
-            unredo_test(self.grid)
+            undo_test(self.grid)
 
         except TypeError:
             assert res == 'fail'
 
 
-class TestUnRedoActions(object):
-    """Unit test class for undo and redo actions"""
-
-    def setup_method(self, method):
-        self.main_window = MainWindow(None, -1)
-        self.grid = self.main_window.grid
-        self.code_array = self.grid.code_array
-
-    def test_undo(self):
-        """Tests undo action"""
-
-        restore_basic_grid(self.grid)
-        self.grid.actions.clear()
-
-        self.grid.code_array[(0, 0, 0)] = "Test"
-
-        self.grid.actions.undo()
-
-        assert self.grid.code_array((0, 0, 0)) is None
-
-    def test_redo(self):
-        """Tests redo action"""
-
-        self.test_undo()
-        self.grid.actions.redo()
-
-        assert self.grid.code_array((0, 0, 0)) == "Test"
-
-
 class TestGridActions(object):
     """self.grid level self.grid actions test class"""
 
@@ -908,7 +874,7 @@ class TestGridActions(object):
             event = self.Event()
             event.shape = dim
             self.grid.actions.new(event)
-            new_shape = self.grid.GetTable().data_array.shape
+            new_shape = self.grid.code_array.shape
             assert new_shape == dim
 
     param_zoom_rows = [
@@ -1088,7 +1054,7 @@ class TestSelectionActions(object):
         assert not self.grid.code_array.result_cache
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
     def test_quote_selection(self):
         """Tests for quote_selection"""
@@ -1105,7 +1071,7 @@ class TestSelectionActions(object):
         assert self.grid.code_array((2, 0, 0)) == 'u"NQ1"'
 
         # Test equality of code_array after undo and subsequent redo
-        unredo_test(self.grid)
+        undo_test(self.grid)
 
 
 class TestFindActions(object):
