@@ -29,7 +29,7 @@ from ast import literal_eval
 
 import wx
 
-VERSION = "1.1"
+VERSION = "1.1.1"
 
 
 class DefaultConfig(object):
@@ -54,7 +54,7 @@ class DefaultConfig(object):
         # UI language
         # -----------
 
-        self.ui_language = "'en'"  # 'system' for system locale 
+        self.ui_language = "'en'"  # 'system' for system locale
 
         # Spell checking lamguage
         # -----------------------
@@ -88,8 +88,6 @@ class DefaultConfig(object):
         self.grid_columns = "100"
         self.grid_tables = "3"
 
-        self.max_unredo = "5000"
-
         self.timer_interval = "1000"
 
         # Default row height and col width e.g. for Cairo rendering
@@ -105,6 +103,7 @@ class DefaultConfig(object):
         self.background_color = repr(wx.SYS_COLOUR_WINDOW)
         self.text_color = repr(wx.SYS_COLOUR_WINDOWTEXT)
         self.freeze_color = repr(wx.SYS_COLOUR_HIGHLIGHT)
+        self.label_color = repr(wx.SYS_COLOUR_BTNFACE)
 
         # Fonts
 
@@ -123,6 +122,9 @@ class DefaultConfig(object):
         # Increase and decrease factor on zoom in and zoom out
         self.zoom_factor = "0.05"
 
+        # Mouse wheel scroll speed for the grid
+        self.mouse_wheel_speed_factor = "3"
+
         # GPG parameters
         # --------------
 
