@@ -38,6 +38,7 @@ Provides:
   - PreferencesDialog
   - GPGParamsDialog
   - PasteAsDialog
+  - DependencyDialog
 
 """
 
@@ -797,8 +798,8 @@ class CsvExportDialog(wx.Dialog):
 # end of class CsvImportDialog
 
 
-class MacroDialog(wx.Frame, MainWindowEventMixin):
-    """Macro management dialog"""
+class MacroPanel(wx.Panel, MainWindowEventMixin):
+    """Macro management panel"""
 
     def __init__(self, parent, macros, *args, **kwds):
 
@@ -809,7 +810,7 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         self.parent = parent
         self.macros = macros
 
-        wx.Frame.__init__(self, parent, *args, **kwds)
+        wx.Panel.__init__(self, parent, *args, **kwds)
 
         self.splitter = wx.SplitterWindow(self, -1,
                                           style=wx.SP_3D | wx.SP_BORDER)
@@ -824,9 +825,7 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH
         self.result_ctrl = wx.TextCtrl(self.lower_panel, -1, style=style)
 
-        self.ok_button = wx.Button(self.lower_panel, wx.ID_OK)
         self.apply_button = wx.Button(self.lower_panel, wx.ID_APPLY)
-        self.close_button = wx.Button(self.lower_panel, wx.ID_CLOSE)
 
         self._set_properties()
         self._do_layout()
@@ -835,9 +834,7 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
 
         # Bindings
         self.Bind(stc.EVT_STC_MODIFIED, self.OnText, self.codetext_ctrl)
-        self.Bind(wx.EVT_BUTTON, self.OnOk, self.ok_button)
         self.Bind(wx.EVT_BUTTON, self.OnApply, self.apply_button)
-        self.Bind(wx.EVT_BUTTON, self.OnClose, self.close_button)
         parent.Bind(self.EVT_CMD_MACROERR, self.update_result_ctrl)
 
         # States
@@ -856,16 +853,14 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         upper_sizer.Add(self.codetext_ctrl, 1, wx.EXPAND, 0)
         lower_sizer.Add(self.result_ctrl, 1, wx.EXPAND, 0)
         lower_sizer.Add(button_sizer, 1, wx.EXPAND, 0)
-        button_sizer.Add(self.ok_button, 1, wx.EXPAND, 0)
         button_sizer.Add(self.apply_button, 1, wx.EXPAND, 0)
-        button_sizer.Add(self.close_button, 1, wx.EXPAND, 0)
 
         self.upper_panel.SetSizer(upper_sizer)
         self.lower_panel.SetSizer(lower_sizer)
+        sash_50 = int(round((config["window_size"][1] - 100) * 0.5))
 
         self.splitter.SplitHorizontally(self.upper_panel,
-                                        self.lower_panel,
-                                        400)
+                                        self.lower_panel, sash_50)
         dialog_main_sizer.Add(self.splitter, 1, wx.EXPAND, 0)
         self.SetSizer(dialog_main_sizer)
         self.Layout()
@@ -873,12 +868,8 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
     def _set_properties(self):
         """Setup title, size and tooltips"""
 
-        self.SetTitle(_("Macro list"))
-        self.SetSize((800, 600))
         self.codetext_ctrl.SetToolTipString(_("Enter python code here."))
-        self.ok_button.SetToolTipString(_("Accept all changes"))
         self.apply_button.SetToolTipString(_("Apply changes to current macro"))
-        self.close_button.SetToolTipString(_("Remove current macro"))
         self.splitter.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
         self.result_ctrl.SetMinSize((10, 10))
 
@@ -887,12 +878,6 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
 
         self.macros = self.codetext_ctrl.GetText()
 
-    def OnOk(self, event):
-        """Event handler for Ok button"""
-
-        self._ok_pressed = True
-        self.OnApply(event)
-
     def OnApply(self, event):
         """Event handler for Apply button"""
 
@@ -921,21 +906,6 @@ class MacroDialog(wx.Frame, MainWindowEventMixin):
         event.Skip()
         return success
 
-    def OnClose(self, event):
-        """Event handler for Cancel button"""
-
-        # Warn if any unsaved changes
-        if self.parent.grid.code_array.macros != self.macros:
-            dlg = wx.MessageDialog(
-                self, _("There are changes in the macro editor "
-                        "which have not yet been applied.  Are you sure you "
-                        "wish to close the editor?"), _("Close Editor"),
-                wx.YES_NO | wx.ICON_WARNING)
-            if dlg.ShowModal() == wx.ID_NO:
-                return
-
-        self.Destroy()
-
     def update_result_ctrl(self, event):
         """Update event result following execution by main window"""
 
@@ -1172,7 +1142,8 @@ class AboutDialog(object):
             350, wx.ClientDC(parent))
         info.WebSite = ("http://manns.github.io/pyspread/",
                         _("Pyspread Web site"))
-        info.Developers = ["Martin Manns", "Jason Sexauer", "Vova Kolobok"]
+        info.Developers = ["Martin Manns", "Jason Sexauer", "Vova Kolobok",
+                           "mgunyho"]
         info.DocWriters = ["Martin Manns", "Bosko Markovic"]
         info.Translators = ["Joe Hansen", "Mark Haanen", "Yuri Chornoivan",
                             u"Mario Blättermann", "Christian Kirbach",
@@ -1180,7 +1151,7 @@ class AboutDialog(object):
                             "Enrico Nicoletto", u"Frédéric Marchal",
                             "Philipp Thomas", "Rafael Fontenelle"]
 
-        license_file = open(get_program_path() + "/COPYING", "r")
+        license_file = open(get_program_path() + os.sep + "COPYING", "r")
         license_text = license_file.read()
         license_file.close()
 
@@ -1247,14 +1218,6 @@ class PreferencesDialog(wx.Dialog):
     save_filetypes = ["pys", "pysu", "xls", "all"]
 
     parameters = [
-        ("max_unredo", {
-            "label": _(u"Max. undo steps"),
-            "tooltip": _(u"Maximum number of undo steps"),
-            "widget": wx.lib.intctrl.IntCtrl,
-            "widget_args": [],
-            "widget_kwargs": {"min": 0, "allow_long": True},
-            "prepocessor": int,
-        }),
         ("grid_rows", {
             "label": _(u"Grid rows"),
             "tooltip": _(u"Number of grid rows when starting pyspread"),
@@ -1543,7 +1506,6 @@ class DependencyDialog(wx.Dialog):
     def __init__(self, parent, *args, **kwargs):
         kwargs["style"] = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
         wx.Dialog.__init__(self, parent, *args, **kwargs)
-        self.SetSize((640, -1))
 
         panel = wx.Panel(self, wx.ID_ANY)
 
@@ -1552,7 +1514,7 @@ class DependencyDialog(wx.Dialog):
                          |wx.BORDER_SUNKEN
                          )
         self.list_ctrl.InsertColumn(0, _('Package'), width=125)
-        self.list_ctrl.InsertColumn(1, _('Required version'), width=125)
+        self.list_ctrl.InsertColumn(1, _('Required version'), width=100)
         self.list_ctrl.InsertColumn(2, _('Installed version'), width=125)
         self.list_ctrl.InsertColumn(3, _('Description'), width=250)
 
@@ -1561,6 +1523,9 @@ class DependencyDialog(wx.Dialog):
         sizer.AddGrowableCol(0)
         sizer.Add(self.list_ctrl, 0, wx.ALL|wx.EXPAND, 5)
         panel.SetSizer(sizer)
+        sizer.Fit(self)
+
+        self.SetSize((640, 400))
 
         self._populate()
 
