@@ -48,10 +48,11 @@ from _menubars import ContextMenu
 from _chart_dialog import ChartDialog
 
 import src.lib.i18n as i18n
-from src.sysvars import is_gtk
+from src.sysvars import is_gtk, get_color
 from src.config import config
 
 from src.lib.selection import Selection
+import src.lib.undo as undo
 from src.model.model import CodeArray
 
 from src.actions._grid_actions import AllGridActions
@@ -82,7 +83,8 @@ class Grid(wx.grid.Grid, glr.GridWithLabelRenderersMixin, EventMixin):
         wx.grid.Grid.__init__(self, main_window, *args, **kwargs)
         glr.GridWithLabelRenderersMixin.__init__(self)
 
-        self.SetDefaultCellBackgroundColour(wx.Colour(255, 255, 255, 255))
+        default_bg_color = get_color(config["background_color"])
+        self.SetDefaultCellBackgroundColour(default_bg_color)
 
         # Cursor position on entering selection mode
         self.sel_mode_cursor = None
@@ -93,7 +95,6 @@ class Grid(wx.grid.Grid, glr.GridWithLabelRenderersMixin, EventMixin):
         # Create new grid
         if S is None:
             self.code_array = CodeArray(dimensions)
-            post_command_event(self, self.GridActionNewMsg, shape=dimensions)
         else:
             self.code_array = S
 
@@ -416,8 +417,7 @@ class Grid(wx.grid.Grid, glr.GridWithLabelRenderersMixin, EventMixin):
 
         selection = Selection([], [], [], [], [key])
 
-        self.actions.set_attr("video_volume", new_video_volume, selection,
-                              mark_unredo=False)
+        self.actions.set_attr("video_volume", new_video_volume, selection)
 
     def ForceRefresh(self, *args, **kwargs):
         """Refresh hook"""
@@ -449,7 +449,6 @@ class GridCellEventHandlers(object):
         """Text entry event handler"""
 
         row, col, _ = self.grid.actions.cursor
-
         self.grid.GetTable().SetValue(row, col, event.code)
 
         event.Skip()
@@ -538,7 +537,7 @@ class GridCellEventHandlers(object):
             except KeyError:
                 video_volume = None
 
-            self.grid.actions.set_attr("panel_cell", True, mark_unredo=False)
+            self.grid.actions.set_attr("panel_cell", True)
 
             if video_volume is not None:
                 code = 'vlcpanel_factory("{}", {})'.format(
@@ -586,7 +585,8 @@ class GridCellEventHandlers(object):
     def OnPasteFormat(self, event):
         """Paste format event handler"""
 
-        self.grid.actions.paste_format()
+        with undo.group(_("Paste format")):
+            self.grid.actions.paste_format()
 
         self.grid.ForceRefresh()
         self.grid.update_attribute_toolbar()
@@ -595,7 +595,8 @@ class GridCellEventHandlers(object):
     def OnCellFont(self, event):
         """Cell font event handler"""
 
-        self.grid.actions.set_attr("textfont", event.font)
+        with undo.group(_("Font")):
+            self.grid.actions.set_attr("textfont", event.font)
 
         self.grid.ForceRefresh()
 
@@ -606,7 +607,8 @@ class GridCellEventHandlers(object):
     def OnCellFontSize(self, event):
         """Cell font size event handler"""
 
-        self.grid.actions.set_attr("pointsize", event.size)
+        with undo.group(_("Font size")):
+            self.grid.actions.set_attr("pointsize", event.size)
 
         self.grid.ForceRefresh()
 
@@ -617,18 +619,20 @@ class GridCellEventHandlers(object):
     def OnCellFontBold(self, event):
         """Cell font bold event handler"""
 
-        try:
+        with undo.group(_("Bold")):
             try:
-                weight = getattr(wx, event.weight[2:])
+                try:
+                    weight = getattr(wx, event.weight[2:])
+
+                except AttributeError:
+                    msg = _("Weight {weight} unknown").format(
+                            weight=event.weight)
+                    raise ValueError(msg)
+
+                self.grid.actions.set_attr("fontweight", weight)
 
             except AttributeError:
-                msg = _("Weight {weight} unknown").format(weight=event.weight)
-                raise ValueError(msg)
-
-            self.grid.actions.set_attr("fontweight", weight)
-
-        except AttributeError:
-            self.grid.actions.toggle_attr("fontweight")
+                self.grid.actions.toggle_attr("fontweight")
 
         self.grid.ForceRefresh()
 
@@ -639,18 +643,19 @@ class GridCellEventHandlers(object):
     def OnCellFontItalics(self, event):
         """Cell font italics event handler"""
 
-        try:
+        with undo.group(_("Italics")):
             try:
-                style = getattr(wx, event.style[2:])
+                try:
+                    style = getattr(wx, event.style[2:])
+
+                except AttributeError:
+                    msg = _("Style {style} unknown").format(style=event.style)
+                    raise ValueError(msg)
+
+                self.grid.actions.set_attr("fontstyle", style)
 
             except AttributeError:
-                msg = _("Style {style} unknown").format(style=event.style)
-                raise ValueError(msg)
-
-            self.grid.actions.set_attr("fontstyle", style)
-
-        except AttributeError:
-            self.grid.actions.toggle_attr("fontstyle")
+                self.grid.actions.toggle_attr("fontstyle")
 
         self.grid.ForceRefresh()
 
@@ -661,7 +666,8 @@ class GridCellEventHandlers(object):
     def OnCellFontUnderline(self, event):
         """Cell font underline event handler"""
 
-        self.grid.actions.toggle_attr("underline")
+        with undo.group(_("Underline")):
+            self.grid.actions.toggle_attr("underline")
 
         self.grid.ForceRefresh()
 
@@ -672,7 +678,8 @@ class GridCellEventHandlers(object):
     def OnCellFontStrikethrough(self, event):
         """Cell font strike through event handler"""
 
-        self.grid.actions.toggle_attr("strikethrough")
+        with undo.group(_("Strikethrough")):
+            self.grid.actions.toggle_attr("strikethrough")
 
         self.grid.ForceRefresh()
 
@@ -683,7 +690,8 @@ class GridCellEventHandlers(object):
     def OnCellFrozen(self, event):
         """Cell frozen event handler"""
 
-        self.grid.actions.change_frozen_attr()
+        with undo.group(_("Frozen")):
+            self.grid.actions.change_frozen_attr()
 
         self.grid.ForceRefresh()
 
@@ -694,7 +702,8 @@ class GridCellEventHandlers(object):
     def OnCellLocked(self, event):
         """Cell locked event handler"""
 
-        self.grid.actions.toggle_attr("locked")
+        with undo.group(_("Lock")):
+            self.grid.actions.toggle_attr("locked")
 
         self.grid.ForceRefresh()
 
@@ -708,7 +717,8 @@ class GridCellEventHandlers(object):
         # The button text
         text = event.text
 
-        self.grid.actions.set_attr("button_cell", text, mark_unredo=False)
+        with undo.group(_("Button")):
+            self.grid.actions.set_attr("button_cell", text)
 
         self.grid.ForceRefresh()
 
@@ -719,7 +729,8 @@ class GridCellEventHandlers(object):
     def OnCellMarkup(self, event):
         """Cell markup event handler"""
 
-        self.grid.actions.toggle_attr("markup")
+        with undo.group(_("Markup")):
+            self.grid.actions.toggle_attr("markup")
 
         self.grid.ForceRefresh()
 
@@ -730,7 +741,8 @@ class GridCellEventHandlers(object):
     def OnMerge(self, event):
         """Merge cells event handler"""
 
-        self.grid.actions.merge_selected_cells(self.grid.selection)
+        with undo.group(_("Merge cells")):
+            self.grid.actions.merge_selected_cells(self.grid.selection)
 
         self.grid.ForceRefresh()
 
@@ -739,7 +751,8 @@ class GridCellEventHandlers(object):
     def OnCellJustification(self, event):
         """Horizontal cell justification event handler"""
 
-        self.grid.actions.toggle_attr("justification")
+        with undo.group(_("Justification")):
+            self.grid.actions.toggle_attr("justification")
 
         self.grid.ForceRefresh()
 
@@ -750,7 +763,8 @@ class GridCellEventHandlers(object):
     def OnCellAlignment(self, event):
         """Vertical cell alignment event handler"""
 
-        self.grid.actions.toggle_attr("vertical_align")
+        with undo.group(_("Alignment")):
+            self.grid.actions.toggle_attr("vertical_align")
 
         self.grid.ForceRefresh()
 
@@ -761,8 +775,9 @@ class GridCellEventHandlers(object):
     def OnCellBorderWidth(self, event):
         """Cell border width event handler"""
 
-        self.grid.actions.set_border_attr("borderwidth",
-                                          event.width, event.borders)
+        with undo.group(_("Border width")):
+            self.grid.actions.set_border_attr("borderwidth",
+                                              event.width, event.borders)
 
         self.grid.ForceRefresh()
 
@@ -773,8 +788,9 @@ class GridCellEventHandlers(object):
     def OnCellBorderColor(self, event):
         """Cell border color event handler"""
 
-        self.grid.actions.set_border_attr("bordercolor",
-                                          event.color, event.borders)
+        with undo.group(_("Border color")):
+            self.grid.actions.set_border_attr("bordercolor",
+                                              event.color, event.borders)
 
         self.grid.ForceRefresh()
 
@@ -785,7 +801,8 @@ class GridCellEventHandlers(object):
     def OnCellBackgroundColor(self, event):
         """Cell background color event handler"""
 
-        self.grid.actions.set_attr("bgcolor", event.color)
+        with undo.group(_("Background color")):
+            self.grid.actions.set_attr("bgcolor", event.color)
 
         self.grid.ForceRefresh()
 
@@ -796,7 +813,8 @@ class GridCellEventHandlers(object):
     def OnCellTextColor(self, event):
         """Cell text color event handler"""
 
-        self.grid.actions.set_attr("textcolor", event.color)
+        with undo.group(_("Text color")):
+            self.grid.actions.set_attr("textcolor", event.color)
 
         self.grid.ForceRefresh()
 
@@ -807,7 +825,8 @@ class GridCellEventHandlers(object):
     def OnTextRotation0(self, event):
         """Text rotation dialog event handler"""
 
-        self.grid.actions.set_attr("angle", 0)
+        with undo.group(_("Rotation 0°")):
+            self.grid.actions.set_attr("angle", 0)
 
         self.grid.ForceRefresh()
 
@@ -816,7 +835,8 @@ class GridCellEventHandlers(object):
     def OnTextRotation90(self, event):
         """Text rotation dialog event handler"""
 
-        self.grid.actions.set_attr("angle", 90)
+        with undo.group(_("Rotation 90°")):
+            self.grid.actions.set_attr("angle", 90)
 
         self.grid.ForceRefresh()
 
@@ -825,7 +845,8 @@ class GridCellEventHandlers(object):
     def OnTextRotation180(self, event):
         """Text rotation dialog event handler"""
 
-        self.grid.actions.set_attr("angle", 180)
+        with undo.group(_("Rotation 180°")):
+            self.grid.actions.set_attr("angle", 180)
 
         self.grid.ForceRefresh()
 
@@ -834,7 +855,8 @@ class GridCellEventHandlers(object):
     def OnTextRotation270(self, event):
         """Text rotation dialog event handler"""
 
-        self.grid.actions.set_attr("angle", -90)
+        with undo.group(_("Rotation 270°")):
+            self.grid.actions.set_attr("angle", -90)
 
         self.grid.ForceRefresh()
 
@@ -843,7 +865,8 @@ class GridCellEventHandlers(object):
     def OnCellTextRotation(self, event):
         """Cell text rotation event handler"""
 
-        self.grid.actions.toggle_attr("angle")
+        with undo.group(_("Rotation")):
+            self.grid.actions.toggle_attr("angle")
 
         self.grid.ForceRefresh()
 
@@ -1150,10 +1173,15 @@ class GridEventHandlers(GridActionEventMixin):
             return
 
         else:
+            wheel_speed = config["mouse_wheel_speed_factor"]
             x, y = self.grid.GetViewStart()
-            direction = 1 if event.GetWheelRotation() < 0 else -1
-            self.grid.Scroll(x, y + direction)
-            event.Skip()
+            direction = wheel_speed if event.GetWheelRotation() < 0 \
+                else -wheel_speed
+            if event.ShiftDown():
+                # Scroll sideways if shift is pressed.
+                self.grid.Scroll(x + direction, y)
+            else:
+                self.grid.Scroll(x, y + direction)
 
     # Find events
 
@@ -1233,7 +1261,8 @@ class GridEventHandlers(GridActionEventMixin):
             statustext = statustext.format(find_string=find_string)
 
         else:
-            self.grid.actions.replace(findpos, find_string, replace_string)
+            with undo.group(_("Replace")):
+                self.grid.actions.replace(findpos, find_string, replace_string)
             self.grid.actions.cursor = findpos
 
             # Update statusbar
@@ -1257,8 +1286,9 @@ class GridEventHandlers(GridActionEventMixin):
 
         findpositions = self.grid.actions.find_all(find_string, flags)
 
-        self.grid.actions.replace_all(findpositions, find_string,
-                                      replace_string)
+        with undo.group(_("Replace all")):
+            self.grid.actions.replace_all(findpositions, find_string,
+                                          replace_string)
 
         event.Skip()
 
@@ -1303,7 +1333,8 @@ class GridEventHandlers(GridActionEventMixin):
             ins_point = bbox[0][0] - 1
             no_rows = self._get_no_rowscols(bbox)[0]
 
-        self.grid.actions.insert_rows(ins_point, no_rows)
+        with undo.group(_("Insert rows")):
+            self.grid.actions.insert_rows(ins_point, no_rows)
 
         self.grid.GetTable().ResetView()
 
@@ -1326,7 +1357,8 @@ class GridEventHandlers(GridActionEventMixin):
             ins_point = bbox[0][1] - 1
             no_cols = self._get_no_rowscols(bbox)[1]
 
-        self.grid.actions.insert_cols(ins_point, no_cols)
+        with undo.group(_("Insert columns")):
+            self.grid.actions.insert_cols(ins_point, no_cols)
 
         self.grid.GetTable().ResetView()
 
@@ -1338,7 +1370,9 @@ class GridEventHandlers(GridActionEventMixin):
     def OnInsertTabs(self, event):
         """Insert one table into grid"""
 
-        self.grid.actions.insert_tabs(self.grid.current_table - 1, 1)
+        with undo.group(_("Insert table")):
+            self.grid.actions.insert_tabs(self.grid.current_table - 1, 1)
+
         self.grid.GetTable().ResetView()
         self.grid.actions.zoom()
 
@@ -1358,7 +1392,8 @@ class GridEventHandlers(GridActionEventMixin):
             del_point = bbox[0][0]
             no_rows = self._get_no_rowscols(bbox)[0]
 
-        self.grid.actions.delete_rows(del_point, no_rows)
+        with undo.group(_("Delete rows")):
+            self.grid.actions.delete_rows(del_point, no_rows)
 
         self.grid.GetTable().ResetView()
 
@@ -1381,7 +1416,8 @@ class GridEventHandlers(GridActionEventMixin):
             del_point = bbox[0][1]
             no_cols = self._get_no_rowscols(bbox)[1]
 
-        self.grid.actions.delete_cols(del_point, no_cols)
+        with undo.group(_("Delete columns")):
+            self.grid.actions.delete_cols(del_point, no_cols)
 
         self.grid.GetTable().ResetView()
 
@@ -1393,7 +1429,9 @@ class GridEventHandlers(GridActionEventMixin):
     def OnDeleteTabs(self, event):
         """Deletes tables"""
 
-        self.grid.actions.delete_tabs(self.grid.current_table, 1)
+        with undo.group(_("Delete table")):
+            self.grid.actions.delete_tabs(self.grid.current_table, 1)
+
         self.grid.GetTable().ResetView()
         self.grid.actions.zoom()
 
@@ -1409,9 +1447,8 @@ class GridEventHandlers(GridActionEventMixin):
         if new_shape is None:
             return
 
-        self.grid.actions.change_grid_shape(new_shape)
-
-        self.grid.GetTable().ResetView()
+        with undo.group(_("Resize grid")):
+            self.grid.actions.change_grid_shape(new_shape)
 
         statustext = _("Grid dimensions changed to {shape}.")
         statustext = statustext.format(shape=new_shape)
@@ -1426,22 +1463,23 @@ class GridEventHandlers(GridActionEventMixin):
         grid = self.grid
         grid.DisableCellEditControl()
 
-        # Is a selection present?
-        if self.grid.IsSelection():
-            # Enclose all selected cells
-            self.grid.actions.quote_selection()
+        with undo.group(_("Quote cell(s)")):
+            # Is a selection present?
+            if self.grid.IsSelection():
+                # Enclose all selected cells
+                self.grid.actions.quote_selection()
 
-            # Update grid
-            self.grid.ForceRefresh()
+                # Update grid
+                self.grid.ForceRefresh()
 
-        else:
-            row = self.grid.GetGridCursorRow()
-            col = self.grid.GetGridCursorCol()
-            key = row, col, grid.current_table
+            else:
+                row = self.grid.GetGridCursorRow()
+                col = self.grid.GetGridCursorCol()
+                key = row, col, grid.current_table
 
-            self.grid.actions.quote_code(key)
+                self.grid.actions.quote_code(key)
 
-            grid.MoveCursorDown(False)
+                grid.MoveCursorDown(False)
 
     # Grid attribute events
 
@@ -1466,11 +1504,15 @@ class GridEventHandlers(GridActionEventMixin):
             if leftmost_col == 0 and rightmost_col == num_cols:
                 rows += range(box[0][0], box[1][0]+1)
 
-        for row in rows:
-            self.grid.code_array.set_row_height(row, tab, rowsize,
-                                                mark_unredo=False)
-            self.grid.SetRowSize(row, rowsize * self.grid.grid_renderer.zoom)
-        self.grid.code_array.unredo.mark()
+        # All row resizing is undone in one click
+        with undo.group(_("Resize Rows")):
+            for row in rows:
+                self.grid.code_array.set_row_height(row, tab, rowsize)
+                zoomed_rowsize = rowsize * self.grid.grid_renderer.zoom
+                self.grid.SetRowSize(row, zoomed_rowsize)
+
+        # Mark content as changed
+        post_command_event(self.grid.main_window, self.grid.ContentChangedMsg)
 
         event.Skip()
         self.grid.ForceRefresh()
@@ -1496,11 +1538,15 @@ class GridEventHandlers(GridActionEventMixin):
             if top_row == 0 and bottom_row == num_rows:
                 cols += range(box[0][1], box[1][1]+1)
 
-        for col in cols:
-            self.grid.code_array.set_col_width(col, tab, colsize,
-                                               mark_unredo=False)
-            self.grid.SetColSize(col, colsize * self.grid.grid_renderer.zoom)
-        self.grid.code_array.unredo.mark()
+        # All column resizing is undone in one click
+        with undo.group(_("Resize Columns")):
+            for col in cols:
+                self.grid.code_array.set_col_width(col, tab, colsize)
+                zoomed_colsize = colsize * self.grid.grid_renderer.zoom
+                self.grid.SetColSize(col, zoomed_colsize)
+
+        # Mark content as changed
+        post_command_event(self.grid.main_window, self.grid.ContentChangedMsg)
 
         event.Skip()
         self.grid.ForceRefresh()
@@ -1509,7 +1555,8 @@ class GridEventHandlers(GridActionEventMixin):
         """Sort ascending event handler"""
 
         try:
-            self.grid.actions.sort_ascending(self.grid.actions.cursor)
+            with undo.group(_("Sort ascending")):
+                self.grid.actions.sort_ascending(self.grid.actions.cursor)
             statustext = _(u"Sorting complete.")
 
         except Exception, err:
@@ -1522,7 +1569,8 @@ class GridEventHandlers(GridActionEventMixin):
         """Sort descending event handler"""
 
         try:
-            self.grid.actions.sort_descending(self.grid.actions.cursor)
+            with undo.group(_("Sort descending")):
+                self.grid.actions.sort_descending(self.grid.actions.cursor)
             statustext = _(u"Sorting complete.")
 
         except Exception, err:
@@ -1536,25 +1584,75 @@ class GridEventHandlers(GridActionEventMixin):
     def OnUndo(self, event):
         """Calls the grid undo method"""
 
-        self.grid.actions.undo()
-        self.grid.GetTable().ResetView()
-        self.grid.Refresh()
+        statustext = undo.stack().undotext()
+        undo.stack().undo()
+
+        # Update content changed state
+        try:
+            post_command_event(self.grid.main_window,
+                               self.grid.ContentChangedMsg)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
+        self.grid.code_array.result_cache.clear()
+
+        post_command_event(self.grid.main_window, self.grid.TableChangedMsg,
+                           updated_cell=True)
+
         # Reset row heights and column widths by zooming
         self.grid.actions.zoom()
+
+        # Change grid table dimensions
+        self.grid.GetTable().ResetView()
+
+        # Update TableChoiceIntCtrl
+        shape = self.grid.code_array.shape
+        post_command_event(self.main_window, self.grid.ResizeGridMsg,
+                           shape=shape)
+
         # Update toolbars
         self.grid.update_entry_line()
         self.grid.update_attribute_toolbar()
 
+        post_command_event(self.grid.main_window, self.grid.StatusBarMsg,
+                           text=statustext)
+
     def OnRedo(self, event):
         """Calls the grid redo method"""
 
-        self.grid.actions.redo()
-        self.grid.GetTable().ResetView()
-        self.grid.Refresh()
+        statustext = undo.stack().redotext()
+        undo.stack().redo()
+
+        # Update content changed state
+        try:
+            post_command_event(self.grid.main_window,
+                               self.grid.ContentChangedMsg)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
+        self.grid.code_array.result_cache.clear()
+
+        post_command_event(self.grid.main_window, self.grid.TableChangedMsg,
+                           updated_cell=True)
+
         # Reset row heights and column widths by zooming
         self.grid.actions.zoom()
+
+        # Change grid table dimensions
+        self.grid.GetTable().ResetView()
+
+        # Update TableChoiceIntCtrl
+        shape = self.grid.code_array.shape
+        post_command_event(self.main_window, self.grid.ResizeGridMsg,
+                           shape=shape)
+
         # Update toolbars
         self.grid.update_entry_line()
         self.grid.update_attribute_toolbar()
 
+        post_command_event(self.grid.main_window, self.grid.StatusBarMsg,
+                           text=statustext)
+
 # End of class GridEventHandlers
