@@ -71,6 +71,7 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
         self._tc.Show(not locked)
 
         if locked:
+            grid.code_array.result_cache.clear()
             self._execute_cell_code(key[0], key[1], grid)
 
         self._tc.SetInsertionPoint(0)
@@ -141,6 +142,7 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
         self._tc.Show(not locked)
 
         if locked:
+            grid.code_array.result_cache.clear()
             self._execute_cell_code(row, col, grid)
 
         # Mirror our changes onto the main_window's code bar
@@ -175,9 +177,18 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
         self._update_control_length()
 
         self._tc.SetInsertionPointEnd()
+
+        h_scroll_pos = grid.GetScrollPos(wx.SB_HORIZONTAL)
+        v_scroll_pos = grid.GetScrollPos(wx.SB_VERTICAL)
+
         self._tc.SetFocus()
 
-        # For this example, select the text
+        # GTK Bugfix for jumping grid when focusing the textctrl
+        if grid.GetScrollPos(wx.SB_HORIZONTAL) != h_scroll_pos or \
+           grid.GetScrollPos(wx.SB_VERTICAL) != v_scroll_pos:
+            wx.ScrolledWindow.Scroll(grid, (h_scroll_pos, v_scroll_pos))
+
+        # Select the text
         self._tc.SetSelection(0, self._tc.GetLastPosition())
 
     def EndEdit(self, row, col, grid, oldVal=None):
@@ -207,6 +218,7 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
         a non-None value.
         *Must Override*
         """
+
         val = self._tc.GetValue()
         grid.GetTable().SetValue(row, col, val)  # update the table
 
@@ -299,9 +311,7 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
 
     def OnChar(self, event):
         self._update_control_length()
-        val = self._tc.GetValue()
-        post_command_event(self.main_window, self.TableChangedMsg,
-                           updated_cell=val)
+
         event.Skip()
 
     def OnKeyUp(self, event):
@@ -309,6 +319,8 @@ class GridCellEditor(wx.grid.PyGridCellEditor, GridEventMixin):
         keycode = event.GetKeyCode()
         if keycode == 13 and event.ControlDown():
             self._tc.SetValue(quote(self._tc.GetValue()))
+        post_command_event(self.main_window, self.TableChangedMsg,
+                           updated_cell=self._tc.GetValue())
         event.Skip()
 
     def _update_control_length(self):
