@@ -48,17 +48,18 @@ except ImportError:
 
 import src.lib.i18n as i18n
 from src.config import config
-from src.sysvars import get_python_tutorial_path, is_gtk
+from src.sysvars import get_python_tutorial_path, is_gtk, get_color
 
 from src.gui._menubars import MainMenu
 from src.gui._toolbars import MainToolbar, MacroToolbar, FindToolbar
 from src.gui._toolbars import WidgetToolbar, AttributesToolbar
 from src.gui._widgets import EntryLineToolbarPanel, StatusBar
 from src.gui._widgets import TableChoiceListCtrl
-from src.gui._dialogs import DependencyDialog
+from src.gui._dialogs import DependencyDialog, MacroPanel
 
 from src.lib.clipboard import Clipboard
 from src.lib.filetypes import get_filetypes2wildcards
+import src.lib.undo as undo
 
 from src.gui._gui_interfaces import GuiInterfaces
 from src.gui.icons import icons
@@ -149,6 +150,10 @@ class MainWindow(wx.Frame, EventMixin):
         # TableChoiceListCtrl
         self.table_list_panel = TableChoiceListCtrl(self, self.grid)
 
+        # Macro panel
+        macros = self.grid.code_array.macros
+        self.macro_panel = MacroPanel(self, macros, -1)
+
         # Clipboard
         self.clipboard = Clipboard()
 
@@ -162,11 +167,26 @@ class MainWindow(wx.Frame, EventMixin):
         self._do_layout()
         self._bind()
 
+        if is_gtk():
+            try:
+                wx.Yield()
+            except:
+                pass
+
+        # Update undo stack savepoint
+        undo.stack().savepoint()
+
+        # Update content changed state
+        try:
+            post_command_event(self.grid.main_window,
+                               self.grid.ContentChangedMsg)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
     def _states(self):
         """Sets main window states"""
 
-        # Has the current file been changed since the last save?
-        self.changed_since_save = False
         self.filepath = None
 
         # Print data
@@ -192,6 +212,7 @@ class MainWindow(wx.Frame, EventMixin):
         toggles = [
             (self.main_toolbar, "main_window_toolbar", _("Main toolbar")),
             (self.macro_toolbar, "macro_toolbar", _("Macro toolbar")),
+            (self.macro_panel, "macro_panel", _("Macro panel")),
             (self.attributes_toolbar, "attributes_toolbar",
              _("Format toolbar")),
             (self.find_toolbar, "find_toolbar", _("Find toolbar")),
@@ -216,6 +237,11 @@ class MainWindow(wx.Frame, EventMixin):
     def _do_layout(self):
         """Adds widgets to the aui manager and controls the layout"""
 
+        # Set background color for the toolbar via the manager
+        ap = self._mgr.GetArtProvider()
+        ap.SetColour(aui.AUI_DOCKART_BACKGROUND_GRADIENT_COLOUR,
+                     get_color(config["background_color"]))
+
         # Add the toolbars to the manager
 
         self._mgr.AddPane(self.main_toolbar, aui.AuiPaneInfo().
@@ -249,6 +275,11 @@ class MainWindow(wx.Frame, EventMixin):
                           Name("table_list_panel").Caption(_("Table")).
                           CenterPane().Left().BestSize(50, 300))
 
+        self._mgr.AddPane(self.macro_panel, aui.AuiPaneInfo().
+                          Name("macro_panel").Caption(_("Macro panel")).
+                          Gripper(False).CenterPane().Right().
+                          BestSize(200, 200))
+
         # Load perspective from config
         window_layout = config["window_layout"]
 
@@ -316,6 +347,8 @@ class MainWindow(wx.Frame, EventMixin):
                   handlers.OnFindToolbarToggle)
         self.Bind(self.EVT_CMD_ENTRYLINE_TOGGLE,
                   handlers.OnEntryLineToggle)
+        self.Bind(self.EVT_CMD_MACROPANEL_TOGGLE,
+                  handlers.OnMacroPanelToggle)
         self.Bind(self.EVT_CMD_TABLELIST_TOGGLE,
                   handlers.OnTableListToggle)
         self.Bind(aui.EVT_AUI_PANE_CLOSE, handlers.OnPaneClose)
@@ -366,7 +399,6 @@ class MainWindow(wx.Frame, EventMixin):
         self.Bind(self.EVT_CMD_DEPENDENCIES, handlers.OnDependencies)
         self.Bind(self.EVT_CMD_ABOUT, handlers.OnAbout)
 
-        self.Bind(self.EVT_CMD_MACROLIST, handlers.OnMacroList)
         self.Bind(self.EVT_CMD_MACROREPLACE, handlers.OnMacroReplace)
         self.Bind(self.EVT_CMD_MACROEXECUTE, handlers.OnMacroExecute)
         self.Bind(self.EVT_CMD_MACROLOAD, handlers.OnMacroListLoad)
@@ -456,11 +488,11 @@ class MainWindowEventHandlers(EventMixin):
     def OnContentChanged(self, event):
         """Titlebar star adjustment event handler"""
 
-        self.main_window.changed_since_save = event.changed
+        self.main_window.grid.update_attribute_toolbar()
 
         title = self.main_window.GetTitle()
 
-        if event.changed:
+        if undo.stack().haschanged():
             # Put * in front of title
             if title[:2] != "* ":
                 new_title = "* " + title
@@ -509,7 +541,7 @@ class MainWindowEventHandlers(EventMixin):
 
         # If changes have taken place save of old grid
 
-        if self.main_window.changed_since_save:
+        if undo.stack().haschanged():
             save_choice = self.interfaces.get_save_request_from_user()
 
             if save_choice is None:
@@ -588,7 +620,6 @@ class MainWindowEventHandlers(EventMixin):
                 _("Key selection failed."))
         else:
             # gnupg is present
-            config["gpg_key_fingerprint"] = repr("")
             genkey()
 
     # Toolbar events
@@ -674,6 +705,16 @@ class MainWindowEventHandlers(EventMixin):
 
         event.Skip()
 
+    def OnMacroPanelToggle(self, event):
+        """Macro panel toggle event handler"""
+
+        entry_line_panel_info = \
+            self.main_window._mgr.GetPane("macro_panel")
+
+        self._toggle_pane(entry_line_panel_info)
+
+        event.Skip()
+
     def OnTableListToggle(self, event):
         """Table list toggle event handler"""
 
@@ -710,7 +751,7 @@ class MainWindowEventHandlers(EventMixin):
 
         # If changes have taken place save of old grid
 
-        if self.main_window.changed_since_save:
+        if undo.stack().haschanged():
             save_choice = self.interfaces.get_save_request_from_user()
 
             if save_choice is None:
@@ -769,10 +810,13 @@ class MainWindowEventHandlers(EventMixin):
             except:
                 pass
 
-        # Mark content as unchanged
+        # Update undo stack savepoint and clear undo stack
+        undo.stack().clear()
+        undo.stack().savepoint()
+
+        # Update content changed state
         try:
-            post_command_event(self.main_window, self.ContentChangedMsg,
-                               changed=False)
+            post_command_event(self.main_window, self.ContentChangedMsg)
         except TypeError:
             # The main window does not exist any more
             pass
@@ -781,8 +825,7 @@ class MainWindowEventHandlers(EventMixin):
         """File open event handler"""
 
         # If changes have taken place save of old grid
-
-        if self.main_window.changed_since_save:
+        if undo.stack().haschanged():
             save_choice = self.interfaces.get_save_request_from_user()
 
             if save_choice is None:
@@ -842,14 +885,18 @@ class MainWindowEventHandlers(EventMixin):
             except:
                 pass
 
-        # Mark content as unchanged
+        # Update savepoint and clear the undo stack
+        undo.stack().clear()
+        undo.stack().savepoint()
+
+        # Update content changed state
         try:
-            post_command_event(self.main_window, self.ContentChangedMsg,
-                               changed=False)
+            post_command_event(self.main_window, self.ContentChangedMsg)
         except TypeError:
             # The main window does not exist any more
             pass
 
+
     def OnSave(self, event):
         """File save event handler"""
 
@@ -888,6 +935,9 @@ class MainWindowEventHandlers(EventMixin):
                            attr={"filepath": self.main_window.filepath,
                                  "filetype": filetype})
 
+        # Update undo stack savepoint
+        undo.stack().savepoint()
+
         # Display file save in status bar
 
         statustext = self.main_window.filepath.split("/")[-1] + " saved."
@@ -1199,7 +1249,10 @@ class MainWindowEventHandlers(EventMixin):
 
         if wx.Window.FindFocus() != entry_line:
             selection = self.main_window.grid.selection
-            data = self.main_window.actions.cut(selection)
+
+            with undo.group(_("Cut")):
+                data = self.main_window.actions.cut(selection)
+
             self.main_window.clipboard.set_clipboard(data)
 
             self.main_window.grid.ForceRefresh()
@@ -1258,7 +1311,8 @@ class MainWindowEventHandlers(EventMixin):
             # We got a grid selection
             key = self.main_window.grid.actions.cursor
 
-            self.main_window.actions.paste(key, data)
+            with undo.group(_("Paste")):
+                self.main_window.actions.paste(key, data)
 
         self.main_window.grid.ForceRefresh()
 
@@ -1270,7 +1324,8 @@ class MainWindowEventHandlers(EventMixin):
         data = self.main_window.clipboard.get_clipboard()
         key = self.main_window.grid.actions.cursor
 
-        self.main_window.actions.paste_as(key, data)
+        with undo.group(_("Paste As...")):
+            self.main_window.actions.paste_as(key, data)
 
         self.main_window.grid.ForceRefresh()
 
@@ -1384,13 +1439,6 @@ class MainWindowEventHandlers(EventMixin):
 
     # Macro events
 
-    def OnMacroList(self, event):
-        """Macro list dialog event handler"""
-
-        self.main_window.interfaces.display_macros()
-
-        event.Skip()
-
     def OnMacroReplace(self, event):
         """Macro change event handler"""
 
