@@ -46,6 +46,7 @@ import src.lib.i18n as i18n
 
 from src.config import config
 from src.sysvars import get_default_font, get_font_list
+import src.lib.undo as undo
 from icons import icons
 from _gui_interfaces import ModalDialogInterfaceMixin
 
@@ -203,6 +204,7 @@ class MainToolbar(ToolbarBase):
         # --------
 
         self.Bind(wx.EVT_TOOL, self.OnToggleTool)
+        self.parent.Bind(self.EVT_CMD_TOOLBAR_UPDATE, self.OnUpdate)
 
     def OnToggleTool(self, event):
         """Tool event handler"""
@@ -220,6 +222,36 @@ class MainToolbar(ToolbarBase):
 
         event.Skip()
 
+    def OnUpdate(self, event):
+        """Updates the toolbar states"""
+
+        # Gray out undo and redo id not available
+
+        undo_toolid = self.label2id["Undo"]
+        redo_toolid = self.label2id["Redo"]
+
+        self.EnableTool(undo_toolid, undo.stack().canundo())
+        self.EnableTool(redo_toolid, undo.stack().canredo())
+
+        # Set ToolTip strings to potential next undo / redo action
+        undotext = undo.stack().undotext()
+        undo_tool = self.FindTool(undo_toolid)
+        if undotext is None:
+            undo_tool.SetShortHelp(_("No undo actions available"))
+        else:
+            undo_tool.SetShortHelp(undotext)
+
+        redotext = undo.stack().redotext()
+        redo_tool = self.FindTool(redo_toolid)
+        if redotext is None:
+            redo_tool.SetShortHelp(_("No redo actions available"))
+        else:
+            redo_tool.SetShortHelp(redotext)
+
+        self.Refresh()
+
+        event.Skip()
+
 # end of class MainToolbar
 
 
