@@ -423,7 +423,7 @@ class ImageComboBox(wx.combo.OwnerDrawnComboBox):
         # Otherwise, draw every other background with
         # different color.
 
-        bg_color = wx.Colour(240, 240, 250)
+        bg_color = get_color(wx.SYS_COLOUR_BACKGROUND)
         dc.SetBrush(wx.Brush(bg_color))
         dc.SetPen(wx.Pen(bg_color))
         dc.DrawRectangleRect(rect)
@@ -593,7 +593,9 @@ class FontChoiceCombobox(ImageComboBox):
         layout.set_font_description(font)
 
         layout.set_text(fontname)
-        context.set_source_rgb(0, 0, 0)
+
+        color = tuple([c / 255.0 for c in get_color(config["text_color"]).Get()])
+        context.set_source_rgb(*color)
 
         height = layout.get_pixel_extents()[1][3]
 
@@ -849,10 +851,10 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
     def OnContentChange(self, event):
         """Event handler for updating the content"""
 
-        if event.text is None:
-            self.SetValue(u"")
-        else:
-            self.SetValue(event.text)
+        self.ignore_changes = True
+        self.SetValue(u"" if event.text is None else event.text)
+
+        self.ignore_changes = False
 
         event.Skip()
 
@@ -1013,12 +1015,14 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
         if hasattr(event, 'updated_cell'):
             # Event posted by cell edit widget.  Even more up to date
             #  than the current cell's contents
+            self.ignore_changes = True
             try:
                 self.SetValue(event.updated_cell)
 
             except TypeError:
                 # None instead of string present
                 pass
+            self.ignore_changes = False
 
         else:
             current_cell = self.main_window.grid.actions.cursor
@@ -1064,7 +1068,7 @@ class StatusBar(wx.StatusBar, StatusBarEventMixin, MainWindowEventMixin):
     def OnMessage(self, event):
         """Statusbar message event handler"""
 
-        self.SetStatusText(event.text)
+        self.SetStatusText(unicode(event.text))
 
     def OnSize(self, evt):
         self.Reposition()  # for normal size events
@@ -1110,7 +1114,8 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
         self.main_window = main_window
         self.no_tabs = no_tabs
 
-        IntCtrl.__init__(self, parent, allow_long=True, style=wx.NO_BORDER)
+        IntCtrl.__init__(self, parent, allow_long=True, style=wx.NO_BORDER,
+                         default_color=get_color(config["text_color"]))
 
         self.last_change_s = time.clock()
 
@@ -1374,7 +1379,6 @@ class TableChoiceListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin, GridEventMixin,
                     # 1. gather item data
                     # 2. delete item in list
                     # 3. insert item & it's data into the list at the new index
-                    print self.startIndex, self.dropIndex
 
                     # dropList is a list of field values from the list control
                     dropList = []
