@@ -40,6 +40,7 @@ import datetime
 import i18n
 import warnings
 import types
+from collections import OrderedDict
 
 import wx
 
@@ -129,7 +130,7 @@ class ChartFigure(Figure):
     """Chart figure class with drawing method"""
 
     plot_type_fixed_attrs = {
-        "plot": ["ydata"],
+        "plot": ["xdata", "ydata"],
         "bar": ["left", "height"],
         "boxplot": ["x"],
         "hist": ["x"],
@@ -190,9 +191,11 @@ class ChartFigure(Figure):
         if xdate_format:
             # We have to validate xdate_format. If wrong then bail out.
             try:
+                self.autofmt_xdate()
                 datetime.date(2000, 1, 1).strftime(xdate_format)
 
             except ValueError:
+                self.autofmt_xdate()
                 return
 
             self.__axes.xaxis_date()
@@ -207,26 +210,28 @@ class ChartFigure(Figure):
 
         self.__axes.clear()
 
-        key2setter = {
-            "title": self.__axes.set_title,
-            "xlabel": self.__axes.set_xlabel,
-            "ylabel": self.__axes.set_ylabel,
-            "xscale": self.__axes.set_xscale,
-            "yscale": self.__axes.set_yscale,
-            "xticks": self.__axes.set_xticks,
-            "xtick_labels": self.__axes.set_xticklabels,
-            "xtick_params": self.__axes.tick_params,
-            "yticks": self.__axes.set_yticks,
-            "ytick_labels": self.__axes.set_yticklabels,
-            "ytick_params": self.__axes.tick_params,
-            "xlim": self.__axes.set_xlim,
-            "ylim": self.__axes.set_ylim,
-            "xgrid": self.__axes.xaxis.grid,
-            "ygrid": self.__axes.yaxis.grid,
-            "xdate_format": self._xdate_setter,
-        }
+        key_setter = [
+            ("title", self.__axes.set_title),
+            ("xlabel", self.__axes.set_xlabel),
+            ("ylabel", self.__axes.set_ylabel),
+            ("xscale", self.__axes.set_xscale),
+            ("yscale", self.__axes.set_yscale),
+            ("xticks", self.__axes.set_xticks),
+            ("xtick_labels", self.__axes.set_xticklabels),
+            ("xtick_params", self.__axes.tick_params),
+            ("yticks", self.__axes.set_yticks),
+            ("ytick_labels", self.__axes.set_yticklabels),
+            ("ytick_params", self.__axes.tick_params),
+            ("xlim", self.__axes.set_xlim),
+            ("ylim", self.__axes.set_ylim),
+            ("xgrid", self.__axes.xaxis.grid),
+            ("ygrid", self.__axes.yaxis.grid),
+            ("xdate_format", self._xdate_setter),
+        ]
 
-        for key in sorted(key2setter.keys()):
+        key2setter = OrderedDict(key_setter)
+
+        for key in key2setter:
             if key in axes_data and axes_data[key]:
                 try:
                     kwargs_key = key + "_kwargs"
@@ -267,7 +272,7 @@ class ChartFigure(Figure):
             if x_str in series and \
                len(series[x_str]) != len(series[y_str]):
                 # Wrong length --> ignore xdata
-                series.pop(x_str)
+                series[x_str] = range(len(series[y_str]))
             else:
                 # Solve the problem that the series data may contain utf-8 data
                 series_list = list(series[x_str])
