@@ -185,6 +185,44 @@ class TestSelection(object):
         val = sel + add
         assert val == res
 
+    param_test_and = [
+        {'s1': Selection([], [], [], [], []),
+         's2': Selection([], [], [], [], []),
+         'res': Selection([], [], [], [], [])},
+        {'s1': Selection([], [], [], [], [(0, 0)]),
+         's2': Selection([], [], [], [], []),
+         'res': Selection([], [], [], [], [])},
+        {'s1': Selection([], [], [], [], [(0, 0)]),
+         's2': Selection([], [], [], [], [(0, 0)]),
+         'res': Selection([], [], [], [], [(0, 0)])},
+        {'s1': Selection([], [], [], [], [(0, 0)]),
+         's2': Selection([(0, 0)], [(5, 5)], [], [], []),
+         'res': Selection([], [], [], [], [(0, 0)])},
+        {'s1': Selection([(0, 0)], [(1000, 200)], [], [], []),
+         's2': Selection([(0, 0)], [(1000, 200)], [], [], []),
+         'res': Selection([(0, 0)], [(1000, 200)], [], [], [])},
+        {'s1': Selection([(0, 0)], [(1000, 200)], [], [], []),
+         's2': Selection([(1, 2)], [(10, 20)], [], [], []),
+         'res': [(1, 2), (10, 20)]},
+        {'s1': Selection([(0, 0)], [(1000, 200)], [], [], []),
+         's2': Selection([], [], [(2)], [], []),
+         'res': [(2, 3), (2, 0), (2, 200)]},
+        {'s1': Selection([(0, 0)], [(1000, 200)], [], [], []),
+         's2': Selection([], [], [], [(5)], []),
+         'res': [(0, 5), (1000, 5)]},
+    ]
+
+    @params(param_test_and)
+    def test_and(self, s1, s2, res):
+        """Unit test for __and__"""
+
+        s1_and_s2 = s1 & s2
+        if type(res) is type([]):
+            for cell in res:
+                assert cell in s1_and_s2
+        else:
+            assert s1_and_s2 == res
+
     param_test_insert = [
         {'sel': Selection([], [], [2], [], []),
          'point': 1, 'number': 10, 'axis': 0,
