@@ -54,31 +54,63 @@ from src.config import config
 from src.lib.typechecks import is_slice_like, is_string_like, is_generator_like
 from src.lib.selection import Selection
 
+from src.lib.undo import undoable
+
 import src.lib.charts as charts
 from src.gui.grid_panels import vlcpanel_factory
 
 from src.sysvars import get_color, get_font_string
 
-from unredo import UnRedo
-
 
 class KeyValueStore(dict):
     """Key-Value store in memory. Currently a dict with default value None.
 
     This class represents layer 0 of the model.
 
+    The following methods mave been made undoable:
+    * __setitem__
+    * pop
+
     """
 
+    def __init__(self, default_value=None):
+        dict.__init__(self)
+
+        self.default_value = default_value
+
     def __missing__(self, value):
         """Returns the default value None"""
 
-        return
+        return self.default_value
+
+    @undoable
+    def __setitem__(self, key, value):
+        old_value = self[key]
+        dict.__setitem__(self, key, value)
+
+        yield "__setitem__"
+        # Undo actions
+        if old_value is None:
+            dict.pop(self, key)
+        else:
+            dict.__setitem__(self, key, old_value)
+
+    @undoable
+    def pop(self, key, *args):
+        res = dict.pop(self, key, *args)
+
+        yield "pop", res
+
+        # Undo actions
+        if res is not None:
+            dict.__setitem__(self, key, res)
 
 # End of class KeyValueStore
 
 # -----------------------------------------------------------------------------
 
 
+
 class CellAttributes(list):
     """Stores cell formatting attributes in a list of 3 - tuples
 
@@ -89,11 +121,30 @@ class CellAttributes(list):
     The class provides attribute read access to single cells via __getitem__
     Otherwise it behaves similar to a list.
 
-    Note that for the method undoable_append to work, unredo has to be
-    defined as class attribute.
+    The following methods mave been made undoable:
+    * __setitem__
+    * append
+
+    List methods that may alter the list have been removed
 
     """
 
+    def __init__(self, *args, **kwargs):
+        self.__add__ = None
+        self.__delattr__ = None
+        self.__delitem__ = None
+        self.__delslice__ = None
+        self.__iadd__ = None
+        self.__imul__ = None
+        self.__rmul__ = None
+        self.__setattr__ = None
+        self.__setslice__ = None
+        self.insert = None
+        self.pop = None
+        self.remove = None
+        self.reverse = None
+        self.sort = None
+
     default_cell_attributes = {
         "borderwidth_bottom": 1,
         "borderwidth_right": 1,
@@ -126,21 +177,19 @@ class CellAttributes(list):
     _attr_cache = {}
     _table_cache = {}
 
-    def undoable_append(self, value, mark_unredo=True):
-        """Appends item to list and provides undo and redo functionality"""
-
-        undo_operation = (self.pop, [])
-        redo_operation = (self.undoable_append, [value, mark_unredo])
-
-        self.unredo.append(undo_operation, redo_operation)
-
-        if mark_unredo:
-            self.unredo.mark()
-
-        self.append(value)
+    @undoable
+    def append(self, value):
+        list.append(self, value)
         self._attr_cache.clear()
+        self._table_cache.clear()
 
-        sel, tab, val = value
+        yield "append"
+
+        # Undo actions
+
+        list.pop(self)
+        self._attr_cache.clear()
+        self._table_cache.clear()
 
     def __getitem__(self, key):
         """Returns attribute dict for a single key"""
@@ -174,6 +223,30 @@ class CellAttributes(list):
 
         return result_dict
 
+    @undoable
+    def __setitem__(self, key, value):
+        """Undoable version of list.__setitem__"""
+
+        try:
+            old_value = list.__getitem__(self, key)
+        except IndexError:
+            old_value = None
+
+        list.__setitem__(self, key, value)
+
+        self._attr_cache.clear()
+        self._table_cache.clear()
+
+        yield "__setitem__"
+
+        if old_value is None:
+            self.pop(key)
+        else:
+            list.__setitem__(self, key, old_value)
+
+        self._attr_cache.clear()
+        self._table_cache.clear()
+
     def _len_table_cache(self):
         """Returns the length of the table cache"""
 
@@ -216,10 +289,6 @@ class CellAttributes(list):
         if merge_area:
             return merge_area[0], merge_area[1], tab
 
-    # Allow getting and setting elements in list
-    get_item = list.__getitem__
-    set_item = list.__setitem__
-
 # End of class CellAttributes
 
 
@@ -250,8 +319,14 @@ class DictGrid(KeyValueStore):
 
         self.macros = u""
 
-        self.row_heights = {}  # Keys have the format (row, table)
-        self.col_widths = {}  # Keys have the format (col, table)
+        default_row_height = config["default_row_height"]
+        default_col_width = config["default_col_width"]
+
+        # Keys have the format (row, table)
+        self.row_heights = KeyValueStore(default_value=default_row_height)
+
+        # Keys have the format (col, table)
+        self.col_widths = KeyValueStore(default_value=default_col_width)
 
     def __getitem__(self, key):
 
@@ -290,10 +365,6 @@ class DataArray(object):
     def __init__(self, shape):
         self.dict_grid = DictGrid(shape)
 
-        # Undo and redo management
-        self.unredo = UnRedo()
-        self.dict_grid.cell_attributes.unredo = self.unredo
-
         # Safe mode
         self.safe_mode = False
 
@@ -474,36 +545,17 @@ class DataArray(object):
 
         return self.dict_grid.keys()
 
-    def pop(self, key, mark_unredo=True):
-        """Pops dict_grid with undo and redo support
+    def pop(self, key):
+        """Pops dict_grid
 
         Parameters
         ----------
         key: 3-tuple of Integer
         \tCell key that shall be popped
-        mark_unredo: Boolean, defaults to True
-        \tIf True then an unredo marker is set after the operation
 
         """
 
-        result = self.dict_grid.pop(key)
-
-        # UnRedo support
-
-        if mark_unredo:
-            self.unredo.mark()
-
-        undo_operation = (self.__setitem__, [key, result, mark_unredo])
-        redo_operation = (self.pop, [key, mark_unredo])
-
-        self.unredo.append(undo_operation, redo_operation)
-
-        if mark_unredo:
-            self.unredo.mark()
-
-        # End UnRedo support
-
-        return result
+        return self.dict_grid.pop(key)
 
     # Shape mask
 
@@ -512,44 +564,43 @@ class DataArray(object):
 
         return self.dict_grid.shape
 
-    def _set_shape(self, shape, mark_unredo=True):
+    @undoable
+    def _set_shape(self, shape):
         """Deletes all cells beyond new shape and sets dict_grid shape
 
         Parameters
         ----------
         shape: 3-tuple of Integer
         \tTarget shape for grid
-        mark_unredo: Boolean, defaults to True
-        \tIf True then an unredo marker is set after the operation
 
         """
 
         # Delete each cell that is beyond new borders
 
         old_shape = self.shape
+        deleted_cells = {}
 
         if any(new_axis < old_axis
                for new_axis, old_axis in zip(shape, old_shape)):
             for key in self.dict_grid.keys():
                 if any(key_ele >= new_axis
                        for key_ele, new_axis in zip(key, shape)):
-                    self.pop(key)
+                    deleted_cells[key] = self.pop(key)
 
         # Set dict_grid shape attribute
-
         self.dict_grid.shape = shape
 
-        # UnRedo support
+        self._adjust_rowcol(0, 0, 0)
+        self._adjust_cell_attributes(0, 0, 0)
 
-        undo_operation = (self._set_shape, [old_shape, mark_unredo])
-        redo_operation = (self._set_shape, [shape, mark_unredo])
+        # Undo actions
 
-        self.unredo.append(undo_operation, redo_operation)
+        yield "_set_shape"
 
-        if mark_unredo:
-            self.unredo.mark()
+        self.shape = old_shape
 
-        # End UnRedo support
+        for key in deleted_cells:
+            self[key] = deleted_cells[key]
 
     shape = property(_get_shape, _set_shape)
 
@@ -617,7 +668,7 @@ class DataArray(object):
 
         return self.dict_grid[key]
 
-    def __setitem__(self, key, value, mark_unredo=True):
+    def __setitem__(self, key, value):
         """Accepts index and slice keys
 
         Parameters
@@ -626,8 +677,6 @@ class DataArray(object):
         \tCell key(s) that shall be set
         value: Object (should be Unicode or similar)
         \tCode for cell(s) to be set
-        mark_unredo: Boolean, defaults to True
-        \tIf True then an unredo marker is set after the operation
 
         """
 
@@ -653,35 +702,8 @@ class DataArray(object):
 
         single_keys = product(*single_keys_per_dim)
 
-        unredo_mark = False
-
         for single_key in single_keys:
             if value:
-                # UnRedo support
-
-                old_value = self(key)
-
-                try:
-                    old_value = unicode(old_value, encoding="utf-8")
-                except TypeError:
-                    pass
-
-                # We seem to have double calls on __setitem__
-                # This hack catches them
-
-                if old_value != value:
-
-                    unredo_mark = True
-
-                    undo_operation = (self.__setitem__,
-                                      [key, old_value, mark_unredo])
-                    redo_operation = (self.__setitem__,
-                                      [key, value, mark_unredo])
-
-                    self.unredo.append(undo_operation, redo_operation)
-
-                    # End UnRedo support
-
                 # Never change merged cells
                 merging_cell = \
                     self.cell_attributes.get_merging_cell(single_key)
@@ -695,9 +717,6 @@ class DataArray(object):
                 except (KeyError, TypeError):
                     pass
 
-        if mark_unredo and unredo_mark:
-            self.unredo.mark()
-
     def cell_array_generator(self, key):
         """Generator traversing cells specified in key
 
@@ -732,12 +751,9 @@ class DataArray(object):
 
                 break
 
-    def _shift_rowcol(self, insertion_point, no_to_insert, mark_unredo):
+    def _shift_rowcol(self, insertion_point, no_to_insert):
         """Shifts row and column sizes when a table is inserted or deleted"""
 
-        if mark_unredo:
-            self.unredo.mark()
-
         # Shift row heights
 
         new_row_heights = {}
@@ -750,12 +766,11 @@ class DataArray(object):
                 del_row_heights.append((row, tab))
 
         for row, tab in new_row_heights:
-            self.set_row_height(row, tab, new_row_heights[(row, tab)],
-                                mark_unredo=False)
+            self.set_row_height(row, tab, new_row_heights[(row, tab)])
 
         for row, tab in del_row_heights:
             if (row, tab) not in new_row_heights:
-                self.set_row_height(row, tab, None, mark_unredo=False)
+                self.set_row_height(row, tab, None)
 
         # Shift column widths
 
@@ -769,29 +784,21 @@ class DataArray(object):
                 del_col_widths.append((col, tab))
 
         for col, tab in new_col_widths:
-            self.set_col_width(col, tab, new_col_widths[(col, tab)],
-                               mark_unredo=False)
+            self.set_col_width(col, tab, new_col_widths[(col, tab)])
 
         for col, tab in del_col_widths:
             if (col, tab) not in new_col_widths:
-                self.set_col_width(col, tab, None, mark_unredo=False)
+                self.set_col_width(col, tab, None)
 
-        if mark_unredo:
-            self.unredo.mark()
-
-    def _adjust_rowcol(self, insertion_point, no_to_insert, axis, tab=None,
-                       mark_unredo=True):
+    def _adjust_rowcol(self, insertion_point, no_to_insert, axis, tab=None):
         """Adjusts row and column sizes on insertion/deletion"""
 
         if axis == 2:
-            self._shift_rowcol(insertion_point, no_to_insert, mark_unredo)
+            self._shift_rowcol(insertion_point, no_to_insert)
             return
 
         assert axis in (0, 1)
 
-        if mark_unredo:
-            self.unredo.mark()
-
         cell_sizes = self.col_widths if axis else self.row_heights
         set_cell_size = self.set_col_width if axis else self.set_row_height
 
@@ -806,18 +813,15 @@ class DataArray(object):
                 del_sizes.append((pos, table))
 
         for pos, table in new_sizes:
-            set_cell_size(pos, table, new_sizes[(pos, table)],
-                          mark_unredo=False)
+            set_cell_size(pos, table, new_sizes[(pos, table)])
 
         for pos, table in del_sizes:
             if (pos, table) not in new_sizes:
-                set_cell_size(pos, table, None, mark_unredo=False)
+                set_cell_size(pos, table, None)
 
-        if mark_unredo:
-            self.unredo.mark()
-
-    def _adjust_merge_area(self, attrs, insertion_point, no_to_insert, axis):
-        """Returns an updated merge area
+    def _get_adjusted_merge_area(self, attrs, insertion_point, no_to_insert,
+                                 axis):
+        """Returns updated merge area
 
         Parameters
         ----------
@@ -839,20 +843,47 @@ class DataArray(object):
 
         top, left, bottom, right = attrs["merge_area"]
         selection = Selection([(top, left)], [(bottom, right)], [], [], [])
+
         selection.insert(insertion_point, no_to_insert, axis)
+
         __top, __left = selection.block_tl[0]
         __bottom, __right = selection.block_br[0]
 
         # Adjust merge area if it is beyond the grid shape
         rows, cols, tabs = self.shape
 
-        if __top < 0 or __bottom >= rows or __left < 0 or __right >= cols:
-            attrs["merge_area"] = None
-        else:
-            attrs["merge_area"] = __top, __left, __bottom, __right
+        if __top < 0 and __bottom < 0 or __top >= rows and __bottom >= rows or\
+           __left < 0 and __right < 0 or __left >= cols and __right >= cols:
+            return
+
+        if __top < 0:
+            __top = 0
+
+        if __top >= rows:
+            __top = rows - 1
+
+        if __bottom < 0:
+            __bottom = 0
+
+        if __bottom >= rows:
+            __bottom = rows - 1
+
+        if __left < 0:
+            __left = 0
+
+        if __left >= cols:
+            __left = cols - 1
+
+        if __right < 0:
+            __right = 0
+
+        if __right >= cols:
+            __right = cols - 1
+
+        return __top, __left, __bottom, __right
 
     def _adjust_cell_attributes(self, insertion_point, no_to_insert, axis,
-                                tab=None, cell_attrs=None, mark_unredo=True):
+                                tab=None, cell_attrs=None):
         """Adjusts cell attributes on insertion/deletion
 
         Parameters
@@ -867,15 +898,30 @@ class DataArray(object):
         \tIf given then insertion is limited to this tab for axis < 2
         cell_attrs: List, defaults to []
         \tIf not empty then the given cell attributes replace the existing ones
-        mark_unredo: Boolean, defaults to True
-        \tIf True then an unredo marker is set after the operation
 
         """
 
         def replace_cell_attributes_table(index, new_table):
-            ca = list(self.cell_attributes.get_item(index))
+            """Replaces table in cell_attributes item"""
+
+            ca = list(list.__getitem__(self.cell_attributes, index))
             ca[1] = new_table
-            self.cell_attributes.set_item(index, tuple(ca))
+            self.cell_attributes.__setitem__(index, tuple(ca))
+
+        def get_ca_with_updated_ma(attrs, merge_area):
+            """Returns cell attributes with updated merge area"""
+
+            new_attrs = copy(attrs)
+
+            if merge_area is None:
+                try:
+                    new_attrs.pop("merge_area")
+                except KeyError:
+                    pass
+            else:
+                new_attrs["merge_area"] = merge_area
+
+            return new_attrs
 
         if axis not in range(3):
             raise ValueError("Axis must be in [0, 1, 2]")
@@ -885,21 +931,29 @@ class DataArray(object):
         if cell_attrs is None:
             cell_attrs = []
 
-        # Store existing cell attributes for creating undo operation
-        old_cell_attrs = self.cell_attributes[:]
-
         if cell_attrs:
             self.cell_attributes[:] = cell_attrs
 
         elif axis < 2:
             # Adjust selections on given table
 
-            for selection, table, attrs in self.cell_attributes:
+            ca_updates = {}
+            for i, (selection, table, attrs) in enumerate(
+                                                    self.cell_attributes):
+                selection = copy(selection)
                 if tab is None or tab == table:
                     selection.insert(insertion_point, no_to_insert, axis)
                     # Update merge area if present
-                    self._adjust_merge_area(attrs, insertion_point,
-                                            no_to_insert, axis)
+                    merge_area = self._get_adjusted_merge_area(attrs,
+                                                               insertion_point,
+                                                               no_to_insert,
+                                                               axis)
+                    new_attrs = get_ca_with_updated_ma(attrs, merge_area)
+
+                    ca_updates[i] = selection, table, new_attrs
+
+            for idx in ca_updates:
+                self.cell_attributes[idx] = ca_updates[idx]
 
         elif axis == 2:
             # Adjust tabs
@@ -926,18 +980,6 @@ class DataArray(object):
         self.cell_attributes._attr_cache.clear()
         self.cell_attributes._update_table_cache()
 
-        undo_operation = (self._adjust_cell_attributes,
-                          [insertion_point, -no_to_insert, axis, tab,
-                           old_cell_attrs, mark_unredo])
-        redo_operation = (self._adjust_cell_attributes,
-                          [insertion_point, no_to_insert, axis, tab,
-                           cell_attrs, mark_unredo])
-
-        self.unredo.append(undo_operation, redo_operation)
-
-        if mark_unredo:
-            self.unredo.mark()
-
     def insert(self, insertion_point, no_to_insert, axis, tab=None):
         """Inserts no_to_insert rows/cols/tabs/... before insertion_point
 
@@ -955,8 +997,6 @@ class DataArray(object):
 
         """
 
-        self.unredo.mark()
-
         if not 0 <= axis <= len(self.shape):
             raise ValueError("Axis not in grid dimensions")
 
@@ -979,17 +1019,13 @@ class DataArray(object):
 
         for key in del_keys:
             if key not in new_keys and self(key) is not None:
-                self.pop(key, mark_unredo=False)
+                self.pop(key)
 
-        self._adjust_rowcol(insertion_point, no_to_insert, axis, tab=tab,
-                            mark_unredo=False)
-        self._adjust_cell_attributes(insertion_point, no_to_insert, axis,
-                                     tab, mark_unredo=False)
+        self._adjust_rowcol(insertion_point, no_to_insert, axis, tab=tab)
+        self._adjust_cell_attributes(insertion_point, no_to_insert, axis, tab)
 
         for key in new_keys:
-            self.__setitem__(key, new_keys[key], mark_unredo=False)
-
-        self.unredo.mark()
+            self.__setitem__(key, new_keys[key])
 
     def delete(self, deletion_point, no_to_delete, axis, tab=None):
         """Deletes no_to_delete rows/cols/... starting with deletion_point
@@ -998,8 +1034,6 @@ class DataArray(object):
 
         """
 
-        self.unredo.mark()
-
         if not 0 <= axis < len(self.shape):
             raise ValueError("Axis not in grid dimensions")
 
@@ -1032,25 +1066,18 @@ class DataArray(object):
         # Now re-insert moved keys
 
         for key in new_keys:
-            self.__setitem__(key, new_keys[key], mark_unredo=False)
+            self.__setitem__(key, new_keys[key])
 
         for key in del_keys:
             if key not in new_keys and self(key) is not None:
-                self.pop(key, mark_unredo=False)
+                self.pop(key)
 
-        self._adjust_rowcol(deletion_point, -no_to_delete, axis, tab=tab,
-                            mark_unredo=False)
-        self._adjust_cell_attributes(deletion_point, -no_to_delete, axis,
-                                     tab, mark_unredo=False)
+        self._adjust_rowcol(deletion_point, -no_to_delete, axis, tab=tab)
+        self._adjust_cell_attributes(deletion_point, -no_to_delete, axis)
 
-        self.unredo.mark()
-
-    def set_row_height(self, row, tab, height, mark_unredo=True):
+    def set_row_height(self, row, tab, height):
         """Sets row height"""
 
-        if mark_unredo:
-            self.unredo.mark()
-
         try:
             old_height = self.row_heights.pop((row, tab))
 
@@ -1060,23 +1087,9 @@ class DataArray(object):
         if height is not None:
             self.row_heights[(row, tab)] = float(height)
 
-        # Make undoable
-
-        undo_operation = (self.set_row_height,
-                          [row, tab, old_height, mark_unredo])
-        redo_operation = (self.set_row_height, [row, tab, height, mark_unredo])
-
-        self.unredo.append(undo_operation, redo_operation)
-
-        if mark_unredo:
-            self.unredo.mark()
-
-    def set_col_width(self, col, tab, width, mark_unredo=True):
+    def set_col_width(self, col, tab, width):
         """Sets column width"""
 
-        if mark_unredo:
-            self.unredo.mark()
-
         try:
             old_width = self.col_widths.pop((col, tab))
 
@@ -1086,17 +1099,6 @@ class DataArray(object):
         if width is not None:
             self.col_widths[(col, tab)] = float(width)
 
-        # Make undoable
-
-        undo_operation = (self.set_col_width,
-                          [col, tab, old_width, mark_unredo])
-        redo_operation = (self.set_col_width, [col, tab, width, mark_unredo])
-
-        self.unredo.append(undo_operation, redo_operation)
-
-        if mark_unredo:
-            self.unredo.mark()
-
     # Element access via call
 
     __call__ = __getitem__
@@ -1124,7 +1126,7 @@ class CodeArray(DataArray):
     # Custom font storage
     custom_fonts = {}
 
-    def __setitem__(self, key, value, mark_unredo=True):
+    def __setitem__(self, key, value):
         """Sets cell code and resets result cache"""
 
         # Prevent unchanged cells from being recalculated on cursor movement
@@ -1136,7 +1138,7 @@ class CodeArray(DataArray):
                     ((value is None or value == "") and
                      repr_key not in self.result_cache)
 
-        DataArray.__setitem__(self, key, value, mark_unredo=mark_unredo)
+        DataArray.__setitem__(self, key, value)
 
         if not unchanged:
             # Reset result cache
@@ -1248,7 +1250,7 @@ class CodeArray(DataArray):
                     'vlcpanel_factory': vlcpanel_factory}
         env = self._get_updated_environment(env_dict=env_dict)
 
-        _old_code = self(key)
+        #_old_code = self(key)
 
         # Return cell value if in safe mode
 
@@ -1330,22 +1332,20 @@ class CodeArray(DataArray):
                     pass
 
         # Change back cell value for evaluation from other cells
-        self.dict_grid[key] = _old_code
+        #self.dict_grid[key] = _old_code
 
         if glob_var is not None:
             globals().update({glob_var: result})
 
         return result
 
-    def pop(self, key, mark_unredo=True):
+    def pop(self, key):
         """Pops dict_grid with undo and redo support
 
         Parameters
         ----------
         key: 3-tuple of Integer
         \tCell key that shall be popped
-        mark_unredo: Boolean, defaults to True
-        \tIf True then an unredo marker is set after the operation
 
         """
 
@@ -1355,7 +1355,7 @@ class CodeArray(DataArray):
         except KeyError:
             pass
 
-        return DataArray.pop(self, key, mark_unredo=mark_unredo)
+        return DataArray.pop(self, key)
 
     def reload_modules(self):
         """Reloads modules that are available in cells"""
@@ -1370,7 +1370,7 @@ class CodeArray(DataArray):
     def clear_globals(self):
         """Clears all newly assigned globals"""
 
-        base_keys = ['cStringIO', 'IntType', 'KeyValueStore', 'UnRedo',
+        base_keys = ['cStringIO', 'IntType', 'KeyValueStore', 'undoable',
                      'is_generator_like', 'is_string_like', 'bz2', 'base64',
                      '__package__', 're', 'config', '__doc__', 'SliceType',
                      'CellAttributes', 'product', 'ast', '__builtins__',
