@@ -46,14 +46,13 @@ sys.path.insert(0, TESTPATH)
 sys.path.insert(0, TESTPATH + (os.sep + os.pardir) * 3)
 sys.path.insert(0, TESTPATH + (os.sep + os.pardir) * 2)
 
-from src.lib.testlib import params, pytest_generate_tests, unredotest_model
+from src.lib.testlib import params, pytest_generate_tests, undotest_model
 
 from src.model.model import KeyValueStore, CellAttributes, DictGrid
 from src.model.model import DataArray, CodeArray
 
 from src.lib.selection import Selection
-
-from src.model.unredo import UnRedo
+from src.lib.undo import stack as undo_stack
 
 
 class TestKeyValueStore(object):
@@ -82,20 +81,19 @@ class TestCellAttributes(object):
         """Creates empty CellAttributes"""
 
         self.cell_attr = CellAttributes()
-        self.cell_attr.unredo = UnRedo()
+        undo_stack().clear()
 
-    def test_undoable_append(self):
-        """Test undoable_append"""
+    def test_append(self):
+        """Test append"""
 
         selection = Selection([], [], [], [], [(23, 12)])
         table = 0
         attr = {"angle": 0.2}
 
-        self.cell_attr.undoable_append((selection, table, attr))
+        self.cell_attr.append((selection, table, attr))
 
-        # Check if 2 items - the actual action and the marker - have been added
-        assert len(self.cell_attr.unredo.undolist) == 2
-        assert len(self.cell_attr.unredo.redolist) == 0
+        # Check if 1 item - the actual action has been added
+        assert undo_stack().undocount() == 1
         assert not self.cell_attr._attr_cache
 
     def test_getitem(self):
@@ -177,7 +175,7 @@ class TestDataArray(object):
 
         assert sorted(self.data_array.keys()) == [(1, 2, 3), (1, 2, 4)]
 
-    @unredotest_model
+    @undotest_model
     def test_pop(self):
         """Unit test for pop"""
 
@@ -193,7 +191,7 @@ class TestDataArray(object):
 
         assert self.data_array.shape == (100, 100, 100)
 
-    @unredotest_model
+    @undotest_model
     def test_set_shape(self):
         """Unit test for _set_shape"""
 
@@ -253,77 +251,75 @@ class TestDataArray(object):
         self.data_array._set_cell_attributes(cell_attributes)
         assert self.data_array.cell_attributes == cell_attributes
 
-    param_adjust_merge_area = [
+    param_get_adjusted_merge_area = [
         {'attrs': {},
          'insertion_point': 0, 'no_to_insert': 1, 'axis': 0,
-         'res_attrs': {}
-         },
-        {'attrs': {'merge_area': None},
-         'insertion_point': 0, 'no_to_insert': 1, 'axis': 0,
-         'res_attrs': {'merge_area': None}
+         'res': None,
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 5, 'no_to_insert': 1, 'axis': 0,
-         'res_attrs': {'merge_area': (2, 2, 3, 4)}
+         'res': (2, 2, 3, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': 1, 'axis': 0,
-         'res_attrs': {'merge_area': (3, 2, 4, 4)}
+         'res': (3, 2, 4, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': 10, 'axis': 0,
-         'res_attrs': {'merge_area': (12, 2, 13, 4)}
+         'res': (12, 2, 13, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 2, 'no_to_insert': 1, 'axis': 0,
-         'res_attrs': {'merge_area': (2, 2, 4, 4)}
+         'res': (2, 2, 4, 4),
          },
         {'attrs': {'merge_area': (992, 2, 993, 4)},
          'insertion_point': 5, 'no_to_insert': 10, 'axis': 0,
-         'res_attrs': {'merge_area': None}
+         'res': None,
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': -1, 'axis': 0,
-         'res_attrs': {'merge_area': (1, 2, 2, 4)}
+         'res': (1, 2, 2, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 5, 'no_to_insert': -1, 'axis': 0,
-         'res_attrs': {'merge_area': (2, 2, 3, 4)}
+         'res': (2, 2, 3, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 2, 'no_to_insert': -1, 'axis': 0,
-         'res_attrs': {'merge_area': (2, 2, 2, 4)}
+         'res': (2, 2, 2, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': -2, 'axis': 0,
-         'res_attrs': {'merge_area': (0, 2, 1, 4)}
+         'res': (0, 2, 1, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': -3, 'axis': 0,
-         'res_attrs': {'merge_area': None}
+         'res': (0, 2, 0, 4),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': 1, 'axis': 1,
-         'res_attrs': {'merge_area': (2, 3, 3, 5)}
+         'res': (2, 3, 3, 5),
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': 200, 'axis': 1,
-         'res_attrs': {'merge_area': None}
+         'res': None,
          },
         {'attrs': {'merge_area': (2, 2, 3, 4)},
          'insertion_point': 0, 'no_to_insert': -2, 'axis': 1,
-         'res_attrs': {'merge_area': (2, 0, 3, 2)}
+         'res': (2, 0, 3, 2),
          },
     ]
 
-    @params(param_adjust_merge_area)
-    def test_adjust_merge_area(self, attrs, insertion_point, no_to_insert,
-                               axis, res_attrs):
-        """Unit test for _adjust_merge_area"""
+    @params(param_get_adjusted_merge_area)
+    def test_get_adjusted_merge_area(self, attrs, insertion_point,
+                                     no_to_insert, axis, res):
+        """Unit test for _get_adjusted_merge_area"""
 
-        self.data_array._adjust_merge_area(attrs, insertion_point,
-                                           no_to_insert, axis)
-        assert attrs == res_attrs
+        merge_area = self.data_array._get_adjusted_merge_area(attrs,
+                                                              insertion_point,
+                                                              no_to_insert,
+                                                              axis)
+        assert merge_area == res
 
     param_adjust_cell_attributes = [
         {'inspoint': 0, 'noins': 5, 'axis': 0,
